# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2016
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock
import testtools
import unittest
from migtools import migration
import pypowervm.entities as ent
from pypowervm import exceptions as pexc
import pypowervm.tests.test_fixtures as fx


class TestMigration(testtools.TestCase):
    """Unit Tests for Instance Migration/Validation."""

    def setUp(self):
        super(TestMigration, self).setUp()
        mock_resp = mock.MagicMock()
        mock_resp.entry = ent.Entry({}, ent.Element('Dummy', None), None)
        self.adpt = self.useFixture(fx.AdapterFx()).adpt
        self.adpt.read.return_value = mock_resp

    @mock.patch('migtools.migration.get_uuid')
    @mock.patch('pypowervm.wrappers.job.Job.run_job')
    @mock.patch('pypowervm.wrappers.job.Job.create_job_parameter')
    @mock.patch('pypowervm.wrappers.managed_system.System')
    def test_validate(self, mock_sys, mock_job_p, mock_run_job, mock_get_uuid):
        mock_get_uuid.return_value = "12345", self.adpt
        migration.validate('Dummy Data')
        self.assertEqual(1, mock_run_job.call_count)
        self.assertEqual(2, mock_job_p.call_count)
        mock_run_job.reset_mock()
        mock_job_p.reset_mock()
        self.adpt.reset_mock()

    @mock.patch('migtools.migration.get_uuid')
    @mock.patch('pypowervm.wrappers.job.Job.run_job')
    @mock.patch('pypowervm.wrappers.job.Job.create_job_parameter')
    @mock.patch('pypowervm.wrappers.managed_system.System')
    def test_validate_failure(self, mock_sys, mock_job_p, mock_run_job,
                              mock_get_uuid):
        """Job request failure"""
        mock_get_uuid.return_value = "12345", self.adpt
        mock_run_job.side_effect = pexc.JobRequestFailed(
            error='Validation failed!', operation_name='validate')
        self.assertRaises(pexc.MigrationFailed, migration.validate,
                          'Dummy Data')

        mock_run_job.reset_mock()
        mock_job_p.reset_mock()
        self.adpt.reset_mock()

        """Job request timed out"""
        mock_run_job.side_effect = pexc.JobRequestTimedOut(
            seconds=1, operation_name='validate')
        self.assertRaises(pexc.MigrationFailed, migration.validate,
                          'Dummy Data')

        mock_run_job.reset_mock()
        mock_job_p.reset_mock()
        self.adpt.reset_mock()

    @mock.patch('pypowervm.wrappers.managed_system.System.wrap')
    @mock.patch('pypowervm.adapter.Adapter')
    def test_get_uuid(self, mock_adapter, mock_system_wrap):
        mock_system_wrap.return_value.uuid = "12345"
        migration.get_uuid()
        mock_adapter.assert_called_with()

if __name__ == "__main__":
    unittest.main()
