# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from pvmcli.command.object_type import ObjectType
import pvmcli.help.help_physical_volume as help_pv
from pvmcli.util.i18n import _
import pvmcli.util.util as util

from pypowervm.wrappers import storage as s
from pypowervm.wrappers import virtual_io_server as v


PRI_DISP_FIELD_LABELS = [_("VIOS"), _("Name"), _("Cap (MB)"), _("State"),
                         _("Description"), _("Available")]


class PhysicalVolume(ObjectType):
    def __init__(self):
        super(PhysicalVolume, self).__init__()

        self.add_display_operation(parent_models=[v.VIOS],
                                   elem_wrp_funcs=[v.VIOS.phys_vols])
        self.add_update_operation(parent_models=[v.VIOS],
                                  elem_wrp_funcs=[v.VIOS.phys_vols])

        self.setup_arguments()

    @property
    def model(self):
        return s.PV

    def _state_str(self, wrp):
        state_val = self.dfield_val(wrp, "PhysicalVolume.state")
        if not state_val:
            return _("invalid")
        if state_val == s.PVState.ACTIVE:
            return _("active")
        elif state_val == s.PVState.MISSING:
            return _("missing")
        elif state_val == s.PVState.REMOVED:
            return _("removed")
        elif state_val == s.PVState.VARIED_OFF:
            return _("varied off")
        else:
            return _("invalid")

    def pv_out_rows(self):
        out_rows = []

        name_to_vios = {}
        for p_w in self.ec.elem_p_wrappers:
            name_to_vios[p_w.name] = p_w

        for vios_name in sorted(name_to_vios):
            p_w = name_to_vios[vios_name]

            name_to_pv = {}
            for wrp in p_w.phys_vols:
                name_to_pv[wrp.name] = wrp

            for pv_name in sorted(name_to_pv):
                wrp = name_to_pv[pv_name]
                out_row = [
                    vios_name, self.dfield_val(wrp, "PhysicalVolume.name"),
                    self.dfield_val(wrp, "PhysicalVolume.capacity"),
                    self._state_str(wrp),
                    self.dfield_val(wrp, "PhysicalVolume.description"),
                    util.bool_str(wrp.avail_for_use)]
                out_rows.append(out_row)
        return out_rows

    def pri_output(self):
        po_table = self.resize_pri_out_table(PRI_DISP_FIELD_LABELS,
                                             self.pv_out_rows())

        return "%(n_ttl)s\n%(n_tbl)s" % {
            'n_ttl': _("Physical Volumes"), 'n_tbl': po_table.get_string()}

    @property
    def help(self):
        return help_pv.help_text
