# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import pvmcli.command.base_partition as bp_c
import pvmcli.help.help_virtual_io_server as help_vios

import pypowervm.wrappers.network as n
import pypowervm.wrappers.storage as s
import pypowervm.wrappers.virtual_io_server as v


class VirtualIOServer(bp_c.BasePartition):
    type_name = "VirtualIOServer"
    title = "Virtual I/O Servers"

    def __init__(self):
        # Exclude pfc_ports display field as the pfc port information is
        # available as part of the ProfileIOSlot.PhysicalFibreChannelAdapter
        exc_dfs = ['VirtualIOServer.pfc_ports']
        super(VirtualIOServer, self).__init__(child_disp_models=[
            n.SEA, n.TrunkAdapter, n.EthernetBackingDevice,
            s.VSCSIClientAdapterElement, s.VSCSIServerAdapterElement,
            s.VFCClientAdapterElement, s.VFCServerAdapterElement, s.LU, s.PV,
            s.VMediaRepos, s.VOptMedia, s.VDisk, v.VSCSIMapping, v.VFCMapping,
            v.LinkAggrIOAdapterChoice, s.LUTargetDev, s.PVTargetDev,
            s.VDiskTargetDev, s.VOptTargetDev], excluded_dfields=exc_dfs)
        self.has_env = False

        self._add_power_on_operation()

        self.setup_arguments()

    @property
    def model(self):
        return v.VIOS

    @property
    def help(self):
        return help_vios.help_text
