# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2016
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import six

from pvmcli.command.object_type import ObjectType
import pvmcli.help.help_virtual_media_repository as help_vmr
from pvmcli.util.i18n import _
import pvmcli.util.util as u

from pypowervm.wrappers import storage as s
from pypowervm.wrappers import virtual_io_server as v


PRI_DISP_FIELD_LABELS = [_("VIOS"), _("VG"), _("VMR Size (GB)"),
                         _("Free (GB)"), _("OM Name"), _("OM Size (GB)"),
                         _("rw")]


def free_repo(vmr):
    ''' Calculate and return the free space for a given media repo. '''
    return vmr.size - sum(vopt.size for vopt in vmr.optical_media)


def _bld_out_row(vios_name, vg_name, vmr):
    b_out_rows = []
    b_out_row = [vios_name, vg_name, six.text_type(vmr.size),
                 six.text_type(free_repo(vmr))]
    if vmr.optical_media:
        for vopt in vmr.optical_media:
            b_out_row.extend([vopt.name, six.text_type(vopt.size),
                              vopt.mount_type])
            b_out_rows.append(b_out_row)
            b_out_row = ["", "", "", ""]
    else:
        b_out_row.extend(["", "", ""])
        b_out_rows.append(b_out_row)
    return b_out_rows


def vmr_out_rows(obj_type, vg_wrps):
    out_rows = []
    vios_to_vg = {}

    for vg_wrp in vg_wrps:
        vios_name = u.vios_for_vg(obj_type.ec.sm.adapter, vg_wrp.href).name
        if vios_name in vios_to_vg:
            vios_to_vg[vios_name].append(vg_wrp)
        else:
            vios_to_vg[vios_name] = [vg_wrp]

    for vios_name in sorted(vios_to_vg):
        for vg in vios_to_vg[vios_name]:
            for vmr in vg.vmedia_repos:
                out_rows.extend(_bld_out_row(vios_name, vg.name, vmr))
    return out_rows


class VirtualMediaRepository(ObjectType):
    def __init__(self):
        super(VirtualMediaRepository, self).__init__()

        self.add_field_models([s.VOptMedia])

        self.add_display_operation(parent_models=[v.VIOS],
                                   child_models=[s.VG],
                                   elem_wrp_funcs=[s.VG.vmedia_repos])

        self.setup_arguments()

    @property
    def model(self):
        return s.VMediaRepos

    def pri_output(self):
        po_table = self.resize_pri_out_table(
            PRI_DISP_FIELD_LABELS, vmr_out_rows(self, self.ec.elem_c_wrappers))

        return "%(n_ttl)s\n%(n_tbl)s" % {
            'n_ttl': _("Virtual Media Repositories"),
            'n_tbl': po_table.get_string()}

    @property
    def help(self):
        return help_vmr.help_text
