# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from pvmcli.command.object_type import ObjectType
import pvmcli.command.operation as op
import pvmcli.command.virtual_media_repository as vmr_c
import pvmcli.help.help_virtual_optical_media as help_vopt
from pvmcli.util import help as help
from pvmcli.util.i18n import _
import pvmcli.util.op_generator as o_g
import pvmcli.util.util as util

from pypowervm.tasks import storage as stor_task
from pypowervm.wrappers import storage as s
from pypowervm.wrappers import virtual_io_server as vios_w


class VirtualOpticalMedia(ObjectType):
    def __init__(self):
        super(VirtualOpticalMedia, self).__init__()

        opt_m = s.VMediaRepos.optical_media
        self.add_display_operation(parent_models=[vios_w.VIOS],
                                   child_models=[s.VG],
                                   elem_wrp_funcs=[s.VG.vmedia_repos, opt_m])
        self._add_upload_operation()
        self.add_delete_operation(parent_models=[vios_w.VIOS],
                                  child_models=[s.VG],
                                  elem_wrp_funcs=[s.VG.vmedia_repos, opt_m],
                                  child_id_req=True)

        self.setup_arguments()

    @property
    def model(self):
        return s.VOptMedia

    def delete(self):
        self.validate_wrps_single_elem(
            self.ec.wrappers, s.VOptMedia.schema_type, op.DELETE_NAME)
        self.validate_wrps_single_elem(
            self.ec.elem_c_wrappers, s.VG.schema_type, op.DELETE_NAME)
        for vmrepo in self.ec.elem_c_wrappers[0].vmedia_repos:
            try:
                vmrepo.optical_media.remove(self.ec.wrappers[0])
            except ValueError:
                # Not in this vmrepo
                pass
        self.ec.elem_c_wrappers[0].update()

    def upload(self):
        data_arg, data_size_arg, vios_uuid = util.upload_setup(self)

        with open(data_arg, mode='rb') as upstream:
            stor_task.upload_vopt(
                self.ec.sm.adapter, vios_uuid, upstream,
                self.ec.parsed_args.name, f_size=data_size_arg,
                sha_chksum=self.ec.parsed_args.checksum)

    def _add_upload_operation(self):
        args = o_g.default_args(self, include_output=False)
        args.extend([op.Argument('--vios', arg_help=help.UPLOAD_VIOS_HELP,
                                 arg_type=str, required=True),
                     op.Argument('--data', arg_help=help.UPLOAD_DATA_HELP,
                                 arg_type=str, required=False),
                     op.Argument('--data-size', arg_help=help.UPLOAD_SIZE_HELP,
                                 arg_type=int, required=False),
                     op.Argument('--name', arg_help=help.UPLOAD_NAME_HELP,
                                 arg_type=str, required=True),
                     op.Argument('--checksum',
                                 arg_help=help.UPLOAD_CHKSUM_HELP,
                                 arg_type=str, required=False)])
        self.add_operation(op.Operation('upload', self.upload, op_args=args,
                                        object_model=self.model,
                                        query_wrps=False))

    def pri_output(self):
        po_table = self.resize_pri_out_table(
            vmr_c.PRI_DISP_FIELD_LABELS,
            vmr_c.vmr_out_rows(self, self.ec.elem_c_wrappers))

        return "%(n_ttl)s\n%(n_tbl)s" % {
            'n_ttl': _("Virtual Optical Media"),
            'n_tbl': po_table.get_string()}

    @property
    def help(self):
        return help_vopt.help_text
