# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2016, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from pvmcli.util.i18n import _
import pvmcli.util.util as util

help_text = _(
    "usage: pvmctl [--version] [help] <object type> [help] <op> [help] "
    "[<args>]\n\nPower Virtualization NovaLink CLI\n\n"
    "pvmctl is a CLI to manage Power Systems through the NovaLink REST "
    "API.\n\n"
    "All CLI commands are driven by operations on object types (listed "
    "below).\nThe available operations are available in the help text for "
    "each object\ntype. For example, `pvmctl vm help` will list "
    "all operations available to\nmanage your logical partitions.\n\n"
    "Operations also have their own help text. For example, `pvmctl vm "
    "create help`\nwill give the help text associated with creating "
    "a new logical partition.\n\n"
    "Object type options (aliases):\n    %(ot_aliases)s\n\n"
    "Optional arguments:\n"
    "    --version             show version of pvmctl\n"
    "    --help | help         show this help message and exit\n")


NL_INDENT = '\n    '


def _alias_str(aliases):
    return (' (' + ', '.join(aliases) + ')') if aliases else ''


def gen_alias_str(o_names, oa_to_name_map):
    '''
    Generate alias string for pvmctl help text based on cli_parser data.

    :param o_names: A list of object type names
    :param oa_to_name_map: A map from alias strs to object type names
    :return: A UTF8 encoded string for object type name and alias help text
    '''
    obj_name_to_aliases = {
        obj_name: [alias for alias in sorted(oa_to_name_map)
                   if oa_to_name_map[alias] == obj_name]
        for obj_name in o_names}
    return util.utf8_str(NL_INDENT.join([o_nm + _alias_str(
        obj_name_to_aliases.get(o_nm)) for o_nm in sorted(o_names)]))
