# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2016, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from pvmcli.util.i18n import _

help_text = _(
    "usage: pvmctl virtualfibrechannelmapping [help] <operation> [help] "
    "[<args>]\n\n"
    "The virtualfibrechannelmapping object type is used to manage all "
    "virtual\nfibre channel mappings and their associated resources.\n\n"
    "Supported operations:\n"
    "    list                  list information about fibre channel mappings\n"
    "    create                create a new VFC mapping\n"
    "    delete                delete an existing VFC mapping\n"
    "    remap                 update an existing VFC mapping with new port"
    "\n\n"
    "Optional arguments:\n"
    "    --help | help         show this help message and exit"
    "\n\n"
    "Object type aliases:\n"
    "    vfc\n"
    "    vfcmapping\n")

_search_args = _(
    """    --parent-id/-p { name=<server name> | id=<server id> }
            Used to identify the server partition of the mapping.
            Example: --parent-id name=myVIOS1

    --lpar { name=<client name> | id=<client id> }
        Used to identify the client partition of the mapping.
        Example: --lpar id=3

    --adpt { name=<adapter name> | udid=<adapter udid> }
        Used to identify the unique server adapter of the mapping.
        Examples: name=vfchost5
                  udid=1dU8247.22L.2125D6A-V2-C11

    --port {name=<port name> | wwpn=<physical wwpn> }
        Used to identify the physical backing port of the mapping.
        Examples: name=fcs1
                  wwpn=21000024ff64912d

    --vios-slot vios_slot_num
        Used to identify the slot number of the mapping for the
        specified server partition.""")

help_create = _(
    """usage: pvmctl virtualfibrechannelmapping create {arguments}

The virtualfibrechannelmapping create operation is used to create a VFC
mapping between a client and server partition.

Required arguments:
    --parent-id/-p { name=<server name> | id=<server id> }
        Used to identify the server partition of the mapping.
        Example: --parent-id name=myVIOS1

    --lpar { name=<client name> | id=<client id> }
        Used to identify the client partition of the mapping.
        Example: --lpar id=3

    --pfcport port_name
        Used to identify the physical backing port of the mapping.
        Example: --pfcport fcs0

Example:
    Create a vfc mapping from client partition with name neo1 to
    server partition with id 2, using pfc port fcs0.
       pvmctl vfc create -p id=2 --lpar name=neo1 --pfcport=fcs0
        ( To find pfc port names available on vios with id 2,
          following command can be used:

          pvmctl vios list -d PhysicalFibreChannelPort.name --where
          VirtualIOServer.name=myVIOS1
        )
""")

help_delete = _(
    """usage: pvmctl virtualfibrechannelmapping delete {arguments}

The virtualfibrechannelmapping delete operation is used to delete a VFC
mapping between a client and server partition.

Possible Search Arguments:
""") + _search_args + _("""

    Arguments are used to help specify a unique mapping to delete. If more
    than one or no mappings are found with the given search args,
    an error will occur and nothing will be deleted.

    Examples:
        Delete vfc mapping under partition ID 5 in LPAR slot 6 from VIOS
        partition with id 2.
            pvmctl vfc delete -p id=2 --lpar id=5 --adpt lpar_slot_num=6

        Delete vfc mapping from server with name vios1 with server adapter name
        of vfchost4
            pvmctl vfc delete -p name=vios1 --adpt name=vfchost4
""")

help_remap = _(
    """usage: pvmctl virtualfibrechannelmapping remap {arguments}

The virtualfibrechannelmapping remap operation is used to modify a VFC
mapping between a client and server partition.

Possible Search Arguments:
""") + _search_args + _("""

Required Arguments:
    --new-port {name=<port name> | wwpn=<physical wwpn> }
        Used to identify the new physical backing port for the mapping.
        Examples: name=fcs1
                  wwpn=21000024ff64912d

    Arguments are used to help specify a unique mapping to update. If more
    than one or no mappings are found with the given search args,
    an error will occur and nothing will be remapped.

Examples:
    Remap a vfc mapping using VIOS slot number 16 to physical port
    named fcs1 under VIOS with id 1
        pvmctl vfc remap -p id=1 --vios-slot 16 --port name=fcs1

    Remap a vfc mapping on VIOS named vios1 with a server adapter UDID
    1dU8247.22L.2125D6A-V2-C17 to physical port with wwpn of
    '21000024ff64912c'
        pvmctl vfc remap -p name=vios1 --adpt udid=1dU8247.22L.2125D6A-V2-C17
        --new-port wwpn=21000024ff64912c
""")
