# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2016, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from pvmcli.util.i18n import _

help_op = _(
    """usage: pvmctl virtualscsimapping [help] <operation> [help] [<args>]

The virtualscsimapping object type is used to manage all virtual SCSI
mappings and their associated resources. A virtual SCSI mapping
connects a disk or optical device from a VIOS to a partition via a pair
of vscsi adapters.

Supported operations:
    create                create a new virtual SCSI mapping
    remap                 modify an existing virtual SCSI mapping
    delete                delete an existing virtual SCSI mapping
    list                  list information about virtual SCSI mappings

Object type aliases:
    scsi
    scsimapping
""")

_type_help = _(
    """    --parent-id/-p { name=<server name> | id=<server id> }
        Used to identify the server partition of the mapping.
        Example: --parent-id name=myVIOS1

    --lpar { name=<client name> | id=<client id> }
        Used to identify the client partition of the mapping.
        Example: --lpar id=3

    --type { file | lu | lv | pv | vopt }
        Used to identify the type of the backing storage.
        Choices:
            file: File I/O
                  File-backed and QCOW2 I/O storage. Specify backing type
                  with --bstype.
            lu: Logical unit
                Cluster/SSP-owned storage. Specify owning SSP with --ssp.
            lv: Logical volume
                Volume group-owned logical storage that may span multiple
                physical volumes. Specify owning volume group with --vg.
            pv: Physical volume
                Physical disk, such as a local SAS disk, FC or iSCSI volume,
                or RAID array.
            vopt: Virtual optical media
                  Optical media-mountable storage, such as an ISO image.

    --stor-id { name | udid | path }=<value>
        <key>=<value> identifier to identify the storage being mapped.
        Storage can be identified by either name or, in some cases,
        udid for all different types of backing storage. In the case of
        file backed mappings, the path key is the preferred mechanism
        for identifying the storage. If no key is specified, the name
        key is used as a default in all cases except the file type,
        which uses path.
        Examples: --stor-id name=myVOPTMedia
                  --stor-id udid=0300025d6a00007a000000015bc487b24b.15
                  --stor-id path=/path/to/my/file.iso""")

_parent_stor_help = _(
    """    --ssp { name | udid }=<value>
        <key>=<value> identifier to identify the SSP that owns the
        logical unit backing storage for this mapping.
        Example: --ssp name=MySSP

    --vg { name | serial_id }=<value>
        <key>=<value> identifier to identify the VG that owns the
        backing storage for this mapping. Used in some cases to
        uniquely identify logical volume and virtual optical media
        storage types.
        Example: --vg name=MyVG"""
)

help_create = _(
    """usage: pvmctl virtualscsimapping create {arguments}

The virtualscsimapping create operation is used to create a mapping
between a client and server partition for existing storage. This
storage can be of type file, logical unit, logical volume, physical
volume or virtual optical media.

Note that some arguments are required in all cases, and some arguments
are optional depending on the type of the backing storage specified in
the --type backing argument.

Required arguments:
""") + _type_help + _("""

Optional arguments:
    --bstype { fileio | user:qcow }
        The backing storage type on a new file I/O mapping. If not
        specified, default to fileio.

""") + _parent_stor_help + _("""

Examples:
    Create a file IO mapping with a path of /my.iso from client
    partition with ID 42 and server partition with name VIOS2.
        pvmctl scsi create -p name=VIOS2 --lpar id=42 --type file
                           --stor-id path=/my.iso

    Create a logical unit mapping from a logical unit with name myLU
    under the SSP mySSP, a server partition of NovaLinkLPAR and a
    client partition of myPart.
        pvmctl scsi create -p name=NovaLinkLPAR --lpar name=myPart
                           --type lu --ssp name=mySSP --stor-id myLU
""")

help_delete = _(
    """usage: pvmctl virtualscsimapping delete {arguments}

The virtualscsimapping delete operation is used to delete a SCSI mapping
between a client and server partition. This mapping may either have backing
storage, or be an orphaned mapping. As such, the mapping identification
arguments depend on whether the mapping is storage-backed or orphaned.

Only one mapping may be deleted per command execution. This requires a unique
identification of that mapping. In the case of a storage-backed mapping,
use a combination of --type and --stor-id, along with other necessary
storage-backed owner arguments (e.g. --vg or --ssp) to make that unique
identification. If the mapping is orphaned, the storage may be identified via
either the vios-slot or the UDID of the server adapter, if available.

Storage-backed mapping arguments:
""") + _type_help + """

""" + _parent_stor_help + _("""

No-storage mapping arguments:
    Note: --udid and --vios-slot are mutually exclusive arguments, used to
          identify the mapping to delete.

    --udid udid
        UDID of the server adapter of the mapping to delete.

    --vios-slot vios-slot
        The VIOS slot number of the mapping to delete.

Examples:
    Delete a file IO mapping with a path of /my.iso from client
    partition with ID 42 and server partition with name VIOS2.
        pvmctl scsi delete -p name=VIOS2 --lpar id=42 --type file
                           --stor-id path=/my.iso

    Delete a storage mapping with server partition with name VIOS2
    on VIOS slot 6, where there is exactly one mapping on the VIOS/slot pair.
        pvmctl scsi delete -p name=VIOS2 --vios-slot 6

    Delete a logical unit mapping with a logical unit with name myLU
    under the SSP mySSP, a server partition of NovaLinkLPAR and a
    client partition of myPart.
        pvmctl scsi delete -p name=NovaLinkLPAR --lpar name=myPart
                           --type lu --ssp name=mySSP --stor-id myLU

    Delete a storage mapping with server partition with name VIOS2
    with server adapter UDID of 1eU8247.22L.123456A-V1-C1.
        pvmctl scsi delete -p name=VIOS2 --udid 1eU8247.22L.123456A-V1-C1
""")
