# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2016, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import pvmcli.command.cluster as cl_c
import pvmcli.help.help_cluster as help_cluster
import pvmcli.tests.util.test_util as test_util

from pypowervm.wrappers import cluster
from pypowervm.wrappers import storage as s
from pypowervm.wrappers import mtms
import pypowervm.tests.test_utils.test_wrapper_abc as twa

LIST_USAGE_HELP = (["usage: pvmctl [help | --help] Cluster list {--object-id "
                    "| -i} <Cluster field>=value {args}"])

DISPLAY_FIELD_HELP = (
    ["--display-fields options:\n"
     "    Cluster.id\n"
     "    Cluster.name\n"
     "    Cluster.repos_pv\n"
     "    Cluster.ssp_uri\n"
     "    Cluster.ssp_uuid\n"
     "    Cluster.uuid\n"
     "    MachineTypeModelAndSerialNumber.machine_type\n"
     "    MachineTypeModelAndSerialNumber.model\n"
     "    MachineTypeModelAndSerialNumber.mtms_str\n"
     "    MachineTypeModelAndSerialNumber.serial\n"
     "    Node.hostname\n"
     "    Node.lpar_id\n"
     "    Node.mtms\n"
     "    Node.state\n"
     "    Node.vios_uri\n"
     "    Node.vios_uuid\n"
     "    PhysicalVolume.avail_for_use\n"
     "    PhysicalVolume.capacity\n"
     "    PhysicalVolume.description\n"
     "    PhysicalVolume.emulate_model\n"
     "    PhysicalVolume.is_fc_backed\n"
     "    PhysicalVolume.name\n"
     "    PhysicalVolume.state\n"
     "    PhysicalVolume.udid\n"])


class ClusterTest(twa.TestWrapper):
    ''' Test framework for Cluster Object Type. '''
    @property
    def file(self):
        return 'cluster.txt'

    @property
    def wrapper_class_to_test(self):
        return cluster.Cluster

    def test_cluster(self):
        cl = cl_c.Cluster()

        test_util.test_obj_type(self, cl, 'Cluster', cluster.Cluster,
                                field_models=[cluster.Cluster, cluster.Node,
                                              s.PV, mtms.MTMS],
                                name_to_ops={'list'},
                                exp_help=help_cluster.help_text)

        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={cl.name_to_ops['list']: None},
            child_model_ops={cl.name_to_ops['list']: None},
            elem_wrp_func_ops={cl.name_to_ops['list']: None})

    def test_help(self):
        ''' Test help arguments for an object type. '''
        test_util.test_help(self, 'Cluster')

    def test_list(self):
        test_util.test_list(
            self, 'Cluster',
            LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
            ["-----------\n"
             "| Cluster |\n"
             "-----------\n"
             "  id=22cfc907d2abf511e4b2d540f2e95daf30\n"
             "  name=neoclust1\n"
             "  ssp_uri=https://9.1.2.3:12443/rest/api/uom/SharedStoragePool/e"
             "357a79a-7a3d-35b6-8405-55ab6a2d0de7\n"
             "  ssp_uuid=e357a79a-7a3d-35b6-8405-55ab6a2d0de7\n"
             "  uuid=17248b3b-9be7-3bc0-9b89-24a285b5d7ac\n"
             "  --------\n"
             "  | Node |\n"
             "  --------\n"
             "    hostname=foo.example.com\n"
             "    lpar_id=2\n"
             "    state=Up\n"
             "    vios_uri=https://9.1.2.3:12443/rest/api/uom/ManagedSystem/"
             "98498bed-c78a-3a4f-b90a-4b715418fcb6/VirtualIOServer/58C9EB1D-7"
             "213-4956-A011-77D43CC4ACCC\n"
             "    vios_uuid=58C9EB1D-7213-4956-A011-77D43CC4ACCC\n"
             "    -----------------------------------\n"
             "    | MachineTypeModelAndSerialNumber |\n"
             "    -----------------------------------\n"
             "      machine_type=8247\n"
             "      model=22L\n"
             "      mtms_str=8247-22L*2125D1A\n"
             "      serial=2125D1A\n"
             "  --------\n"
             "  | Node |\n"
             "  --------\n"
             "    hostname=bar.example.com\n"
             "    lpar_id=1\n"
             "    state=Down\n"
             "    vios_uri=https://9.1.2.3:12443/rest/api/uom/ManagedSystem/"
             "67dca605-3923-34da-bd8f-26a378fc817f/VirtualIOServer/6424120D-C"
             "A95-437D-9C18-10B06F4B3400\n"
             "    vios_uuid=6424120D-CA95-437D-9C18-10B06F4B3400\n"
             "    -----------------------------------\n"
             "    | MachineTypeModelAndSerialNumber |\n"
             "    -----------------------------------\n"
             "      machine_type=8247\n"
             "      model=22L\n"
             "      mtms_str=8247-22L*2125D0A\n"
             "      serial=2125D0A\n"
             "  --------\n"
             "  | Node |\n"
             "  --------\n"
             "    hostname=bar.example.com\n"
             "    lpar_id=3\n"
             "    state=Unknown\n"
             "    vios_uri=https://9.1.2.3:12443/rest/api/uom/ManagedSystem/"
             "67dca605-3923-34da-bd8f-26a378fc817f/VirtualIOServer/10B06F4B-4"
             "37D-9C18-CA95-34006424120D\n"
             "    vios_uuid=10B06F4B-437D-9C18-CA95-34006424120D\n"
             "    -----------------------------------\n"
             "    | MachineTypeModelAndSerialNumber |\n"
             "    -----------------------------------\n"
             "      machine_type=8247\n"
             "      model=22L\n"
             "      mtms_str=8247-22L*2125D0A\n"
             "      serial=2125D0A\n"
             "  ------------------\n"
             "  | PhysicalVolume |\n"
             "  ------------------\n"
             "    avail_for_use=False\n"
             "    capacity=10240\n"
             "    description=MPIO IBM 2076 FC Disk\n"
             "    emulate_model=True\n"
             "    is_fc_backed=True\n"
             "    name=hdisk2\n"
             "    state=active\n"
             "    udid=01M0lCTTIxNDUxMjQ2MDA1MDc2ODAyODI4NjFEODgwMDAwMDAwMDAw"
             "MDAwMg==\n"],
            ["-----------\n"
             "| Cluster |\n"
             "-----------\n"
             "  id=22cfc907d2abf511e4b2d540f2e95daf30\n"
             "  name=neoclust1\n"
             "  ssp_uri=https://9.1.2.3:12443/rest/api/uom/SharedStoragePool/"
             "e357a79a-7a3d-35b6-8405-55ab6a2d0de7\n"
             "  ssp_uuid=e357a79a-7a3d-35b6-8405-55ab6a2d0de7\n"
             "  uuid=17248b3b-9be7-3bc0-9b89-24a285b5d7ac\n"
             "  --------\n"
             "  | Node |\n"
             "  --------\n"
             "    hostname=foo.example.com\n"
             "    lpar_id=2\n"
             "    state=Up\n"
             "    vios_uri=https://9.1.2.3:12443/rest/api/uom/ManagedSystem/"
             "98498bed-c78a-3a4f-b90a-4b715418fcb6/VirtualIOServer/58C9EB1D-7"
             "213-4956-A011-77D43CC4ACCC\n"
             "    vios_uuid=58C9EB1D-7213-4956-A011-77D43CC4ACCC\n"
             "    -----------------------------------\n"
             "    | MachineTypeModelAndSerialNumber |\n"
             "    -----------------------------------\n"
             "      machine_type=8247\n"
             "      model=22L\n"
             "      mtms_str=8247-22L*2125D1A\n"
             "      serial=2125D1A\n"
             "  --------\n"
             "  | Node |\n"
             "  --------\n"
             "    hostname=bar.example.com\n"
             "    lpar_id=1\n"
             "    state=Down\n"
             "    vios_uri=https://9.1.2.3:12443/rest/api/uom/ManagedSystem/"
             "67dca605-3923-34da-bd8f-26a378fc817f/VirtualIOServer/6424120D-C"
             "A95-437D-9C18-10B06F4B3400\n"
             "    vios_uuid=6424120D-CA95-437D-9C18-10B06F4B3400\n"
             "    -----------------------------------\n"
             "    | MachineTypeModelAndSerialNumber |\n"
             "    -----------------------------------\n"
             "      machine_type=8247\n"
             "      model=22L\n"
             "      mtms_str=8247-22L*2125D0A\n"
             "      serial=2125D0A\n"
             "  --------\n"
             "  | Node |\n"
             "  --------\n"
             "    hostname=bar.example.com\n"
             "    lpar_id=3\n"
             "    state=Unknown\n"
             "    vios_uri=https://9.1.2.3:12443/rest/api/uom/ManagedSystem/"
             "67dca605-3923-34da-bd8f-26a378fc817f/VirtualIOServer/10B06F4B-4"
             "37D-9C18-CA95-34006424120D\n"
             "    vios_uuid=10B06F4B-437D-9C18-CA95-34006424120D\n"
             "    -----------------------------------\n"
             "    | MachineTypeModelAndSerialNumber |\n"
             "    -----------------------------------\n"
             "      machine_type=8247\n"
             "      model=22L\n"
             "      mtms_str=8247-22L*2125D0A\n"
             "      serial=2125D0A\n"
             "  ------------------\n"
             "  | PhysicalVolume |\n"
             "  ------------------\n"
             "    avail_for_use=False\n"
             "    capacity=10240\n"
             "    description=MPIO IBM 2076 FC Disk\n"
             "    emulate_model=True\n"
             "    is_fc_backed=True\n"
             "    name=hdisk2\n"
             "    state=active\n"
             "    udid=01M0lCTTIxNDUxMjQ2MDA1MDc2ODAyODI4NjFEODgwMDAwMDAwMDAw"
             "MDAwMg==\n"],
            ['<Cluster:Cluster xmlns:Cluster="http://www.ibm.com'],
            ['Cluster.name', 'Node.hostname'],
            ["name=neoclust1,hostname=foo.example.com\n"
             "name=neoclust1,hostname=bar.example.com\n"
             "name=neoclust1,hostname=bar.example.com\n"],
            ["neoclust1,foo.example.com\n"
             "neoclust1,bar.example.com\n"
             "neoclust1,bar.example.com\n"],
            ["name___neoclust1,hostname___foo.example.com\n"
             "name___neoclust1,hostname___bar.example.com\n"
             "name___neoclust1,hostname___bar.example.com\n"],
            ["name=neoclust1___hostname=foo.example.com\n"
             "name=neoclust1___hostname=bar.example.com\n"
             "name=neoclust1___hostname=bar.example.com\n"],
            ["name=neoclust1,hostname=foo.example.com___"
             "name=neoclust1,hostname=bar.example.com___"
             "name=neoclust1,hostname=bar.example.com\n"],
            ["name___neoclust1___hostname___foo.example.com___"
             "name___neoclust1___hostname___bar.example.com___"
             "name___neoclust1___hostname___bar.example.com\n"],
            'Cluster',
            wrps=[self.dwrap])
