# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2022
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock

import pvmcli.cli_parser as cli_parser
import pvmcli.command.logical_partition as lpar_c
import pvmcli.help.help_lpar as help_lpar
import pvmcli.tests.util.test_util as test_util
import pvmcli.util.exceptions as exc

import pypowervm.exceptions as pvm_exc
import pypowervm.tests.test_utils.test_wrapper_abc as twa
import pypowervm.wrappers.base_partition as bp_w
import pypowervm.wrappers.logical_partition as lpar_w
import pypowervm.wrappers.managed_system as msys_w


LIST_USAGE_HELP = (["usage: pvmctl [help | --help] LogicalPartition list "
                    "{--object-id | -i} <LogicalPartition field>=value "
                    "{args}"])


UPDATE_USAGE_HELP = (["usage: pvmctl [help | --help] LogicalPartition update "
                      "{--object-id | -i} <LogicalPartition field>=value "
                      "{args}"])


DELETE_USAGE_HELP = (["usage: pvmctl [help | --help] LogicalPartition delete "
                      "{--object-id | -i} <LogicalPartition "
                      "field>=value {args}\n"])

DEL_TEST = (["optional arguments:\n "
             "  --reusevsn            Reuse the VSN after deleting the LPAR."])

DISPLAY_FIELD_HELP = ([
    "    AssociatedIOSlot.bus_grp_required\n"
    "    AssociatedIOSlot.description\n"
    "    AssociatedIOSlot.drc_index\n"
    "    AssociatedIOSlot.drc_name\n"
    "    AssociatedIOSlot.io_adapter\n"
    "    AssociatedIOSlot.pc_adpt_id\n"
    "    AssociatedIOSlot.pci_class\n"
    "    AssociatedIOSlot.pci_dev_id\n"
    "    AssociatedIOSlot.pci_mfg_id\n"
    "    AssociatedIOSlot.pci_rev_id\n"
    "    AssociatedIOSlot.pci_subsys_dev_id\n"
    "    AssociatedIOSlot.pci_subsys_vendor_id\n"
    "    AssociatedIOSlot.pci_vendor_id\n"
    "    AssociatedIOSlot.phys_loc\n"
    "    DedicatedProcessorConfiguration.desired\n"
    "    DedicatedProcessorConfiguration.max\n"
    "    DedicatedProcessorConfiguration.min\n"
    "    IOAdapter.description\n"
    "    IOAdapter.dev_name\n"
    "    IOAdapter.dev_type\n"
    "    IOAdapter.drc_name\n"
    "    IOAdapter.id\n"
    "    IOAdapter.phys_loc_code\n"
    "    IOAdapter.udid\n"
    "    LogicalPartition.allow_perf_data_collection\n"
    "    LogicalPartition.assoc_sys_uuid\n"
    "    LogicalPartition.avail_priority\n"
    "    LogicalPartition.bootmode\n"
    "    LogicalPartition.capabilities\n"
    "    LogicalPartition.cur_dram_pmem_volumes\n"
    "    LogicalPartition.cur_pmem_volumes\n"
    "    LogicalPartition.current_secure_boot\n"
    "    LogicalPartition.desig_ipl_src\n"
    "    LogicalPartition.disable_secure_boot\n"
    "    LogicalPartition.env\n"
    "    LogicalPartition.id\n"
    "    LogicalPartition.io_config\n"
    "    LogicalPartition.is_mgmt_partition\n"
    "    LogicalPartition.is_service_partition\n"
    "    LogicalPartition.keylock_pos\n"
    "    LogicalPartition.keystore_kbytes\n"
    "    LogicalPartition.kvm_capable\n"
    "    LogicalPartition.kvm_memory_overhead\n"
    "    LogicalPartition.max_dram_pmem_volumes\n"
    "    LogicalPartition.max_pmem_volumes\n"
    "    LogicalPartition.mem_config\n"
    "    LogicalPartition.migration_state\n"
    "    LogicalPartition.name\n"
    "    LogicalPartition.nvram\n"
    "    LogicalPartition.operating_system\n"
    "    LogicalPartition.partition_uuid\n"
    "    LogicalPartition.pending_proc_compat_mode\n"
    "    LogicalPartition.pending_secure_boot\n"
    "    LogicalPartition.proc_compat_mode\n"
    "    LogicalPartition.proc_config\n"
    "    LogicalPartition.ref_code\n"
    "    LogicalPartition.ref_code_full\n"
    "    LogicalPartition.restrictedio\n"
    "    LogicalPartition.rmc_ip\n"
    "    LogicalPartition.rmc_state\n"
    "    LogicalPartition.srr_enabled\n"
    "    LogicalPartition.state\n"
    "    LogicalPartition.uptime\n"
    "    LogicalPartition.uuid\n"
    "    LogicalPartition.virtual_serial_number\n"
    "    LogicalPartition.virtual_software_tier\n"
    "    PartitionCapabilities.io_dlpar\n"
    "    PartitionCapabilities.mem_dlpar\n"
    "    PartitionCapabilities.proc_dlpar\n"
    "    PartitionIOConfiguration.io_slots\n"
    "    PartitionIOConfiguration.max_virtual_slots\n"
    "    PartitionIOConfiguration.tagged_io\n"
    "    PartitionMemoryConfiguration.ame_enabled\n"
    "    PartitionMemoryConfiguration.current\n"
    "    PartitionMemoryConfiguration.desired\n"
    "    PartitionMemoryConfiguration.exp_factor\n"
    "    PartitionMemoryConfiguration.max\n"
    "    PartitionMemoryConfiguration.min\n"
    "    PartitionMemoryConfiguration.ppt_ratio\n"
    "    PartitionMemoryConfiguration.shared_enabled\n"
    "    PartitionProcessorConfiguration.dedicated_proc_cfg\n"
    "    PartitionProcessorConfiguration.has_dedicated\n"
    "    PartitionProcessorConfiguration.shared_proc_cfg\n"
    "    PartitionProcessorConfiguration.sharing_mode\n"
    "    PhysicalFibreChannelAdapter.description\n"
    "    PhysicalFibreChannelAdapter.dev_name\n"
    "    PhysicalFibreChannelAdapter.dev_type\n"
    "    PhysicalFibreChannelAdapter.drc_name\n"
    "    PhysicalFibreChannelAdapter.fc_ports\n"
    "    PhysicalFibreChannelAdapter.id\n"
    "    PhysicalFibreChannelAdapter.phys_loc_code\n"
    "    PhysicalFibreChannelAdapter.udid\n"
    "    PhysicalFibreChannelPort.loc_code\n"
    "    PhysicalFibreChannelPort.name\n"
    "    PhysicalFibreChannelPort.npiv_available_ports\n"
    "    PhysicalFibreChannelPort.npiv_total_ports\n"
    "    PhysicalFibreChannelPort.udid\n"
    "    PhysicalFibreChannelPort.wwpn\n"
    "    ProfileIOSlot.bus_grp_required\n"
    "    ProfileIOSlot.description\n"
    "    ProfileIOSlot.drc_index\n"
    "    ProfileIOSlot.drc_name\n"
    "    ProfileIOSlot.io_adapter\n"
    "    ProfileIOSlot.pc_adpt_id\n"
    "    ProfileIOSlot.pci_class\n"
    "    ProfileIOSlot.pci_dev_id\n"
    "    ProfileIOSlot.pci_mfg_id\n"
    "    ProfileIOSlot.pci_rev_id\n"
    "    ProfileIOSlot.pci_subsys_dev_id\n"
    "    ProfileIOSlot.pci_subsys_vendor_id\n"
    "    ProfileIOSlot.pci_vendor_id\n"
    "    ProfileIOSlot.phys_loc\n"
    "    ProfileIOSlot.required\n"
    "    SharedProcessorConfiguration.desired_units\n"
    "    SharedProcessorConfiguration.desired_virtual\n"
    "    SharedProcessorConfiguration.max_units\n"
    "    SharedProcessorConfiguration.max_virtual\n"
    "    SharedProcessorConfiguration.min_units\n"
    "    SharedProcessorConfiguration.min_virtual\n"
    "    SharedProcessorConfiguration.pool_id\n"
    "    SharedProcessorConfiguration.uncapped_weight\n"
    "    TaggedIO.alt_load_src\n"
    "    TaggedIO.console\n"
    "    TaggedIO.load_src\n"])


SET_FIELD_HELP = ([
    "    DedicatedProcessorConfiguration.desired\n"
    "    DedicatedProcessorConfiguration.max\n"
    "    DedicatedProcessorConfiguration.min\n"
    "    LogicalPartition.allow_perf_data_collection\n"
    "    LogicalPartition.avail_priority\n"
    "    LogicalPartition.bootmode\n"
    "    LogicalPartition.desig_ipl_src\n"
    "    LogicalPartition.disable_secure_boot\n"
    "    LogicalPartition.is_service_partition\n"
    "    LogicalPartition.keylock_pos\n"
    "    LogicalPartition.keystore_kbytes\n"
    "    LogicalPartition.kvm_capable\n"
    "    LogicalPartition.kvm_memory_overhead\n"
    "    LogicalPartition.name\n"
    "    LogicalPartition.pending_secure_boot\n"
    "    LogicalPartition.proc_compat_mode\n"
    "    LogicalPartition.restrictedio\n"
    "    LogicalPartition.srr_enabled\n"
    "    LogicalPartition.virtual_serial_number\n"
    "    LogicalPartition.virtual_software_tier\n"
    "    PartitionIOConfiguration.max_virtual_slots\n"
    "    PartitionIOConfiguration.tagged_io\n"
    "    PartitionMemoryConfiguration.desired\n"
    "    PartitionMemoryConfiguration.exp_factor\n"
    "    PartitionMemoryConfiguration.max\n"
    "    PartitionMemoryConfiguration.min\n"
    "    PartitionMemoryConfiguration.ppt_ratio\n"
    "    PartitionProcessorConfiguration.sharing_mode\n"
    "    ProfileIOSlot.required\n"
    "    SharedProcessorConfiguration.desired_units\n"
    "    SharedProcessorConfiguration.desired_virtual\n"
    "    SharedProcessorConfiguration.max_units\n"
    "    SharedProcessorConfiguration.max_virtual\n"
    "    SharedProcessorConfiguration.min_units\n"
    "    SharedProcessorConfiguration.min_virtual\n"
    "    SharedProcessorConfiguration.pool_id\n"
    "    SharedProcessorConfiguration.uncapped_weight\n"
    "    TaggedIO.alt_load_src\n"
    "    TaggedIO.console\n"
    "    TaggedIO.load_src\n"])


class LogicalPartitionTest(twa.TestWrapper):
    ''' Test framework for LogicalPartition Object Type. '''
    @property
    def file(self):
        return 'lpar.txt'

    @property
    def wrapper_class_to_test(self):
        return lpar_w.LPAR

    def test_logical_partition(self):
        lpar = lpar_c.LogicalPartition()

        field_models = [lpar_w.LPAR,
                        bp_w.PartitionCapabilities,
                        bp_w.PartitionProcessorConfiguration,
                        bp_w.PartitionMemoryConfiguration,
                        bp_w.SharedProcessorConfiguration,
                        bp_w.DedicatedProcessorConfiguration,
                        bp_w.PartitionIOConfiguration,
                        bp_w.IOSlot,
                        bp_w.IOSlot.AssociatedIOSlot,
                        bp_w.IOAdapter,
                        bp_w.PhysFCAdapter,
                        bp_w.PhysFCPort,
                        bp_w.TaggedIO]
        name_to_ops = {'create', 'list', 'update', 'delete', 'panel',
                       'power-on', 'power-off', 'restart', 'migrate',
                       'migrate-stop', 'network-install', 'migrate-recover',
                       'dumprestart'}

        test_util.test_obj_type(self, lpar, 'LogicalPartition', lpar_w.LPAR,
                                field_models, name_to_ops,
                                exp_help=help_lpar.help_text)

        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={lpar.name_to_ops['list']: None,
                                    lpar.name_to_ops['update']: None,
                                    lpar.name_to_ops['delete']: None,
                                    lpar.name_to_ops['power-on']: None,
                                    lpar.name_to_ops['power-off']: None,
                                    lpar.name_to_ops['restart']: None,
                                    lpar.name_to_ops['dumprestart']: None,
                                    lpar.name_to_ops['migrate']: None,
                                    lpar.name_to_ops['migrate-stop']: None,
                                    lpar.name_to_ops['network-install']: None,
                                    lpar.name_to_ops['migrate-recover']: None,
                                    lpar.name_to_ops['create']:
                                    [msys_w.System]},
            child_model_ops={lpar.name_to_ops['list']: None,
                             lpar.name_to_ops['update']: None,
                             lpar.name_to_ops['delete']: None,
                             lpar.name_to_ops['power-on']: None,
                             lpar.name_to_ops['power-off']: None,
                             lpar.name_to_ops['restart']: None,
                             lpar.name_to_ops['dumprestart']: None,
                             lpar.name_to_ops['migrate']: None,
                             lpar.name_to_ops['migrate-stop']: None,
                             lpar.name_to_ops['network-install']: None,
                             lpar.name_to_ops['migrate-recover']: None,
                             lpar.name_to_ops['create']: None},
            elem_wrp_func_ops={lpar.name_to_ops['list']: None,
                               lpar.name_to_ops['update']: None,
                               lpar.name_to_ops['delete']: None,
                               lpar.name_to_ops['power-on']: None,
                               lpar.name_to_ops['power-off']: None,
                               lpar.name_to_ops['restart']: None,
                               lpar.name_to_ops['dumprestart']: None,
                               lpar.name_to_ops['migrate']: None,
                               lpar.name_to_ops['migrate-stop']: None,
                               lpar.name_to_ops['network-install']: None,
                               lpar.name_to_ops['migrate-recover']: None,
                               lpar.name_to_ops['create']: None})

    def test_list(self):
        test_util.test_list(
            self, 'LogicalPartition',
            LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
            ["Logical Partitions\n"
             "+-------------------------+----+---------------+"
             "------+-----------+----------+-----+-----+-----+\n"
             "|           Name          | ID |     State     |"
             " RMC  |    Env    | Ref Code | Mem | CPU | Ent |\n"
             "+-------------------------+----+---------------+"
             "------+-----------+----------+-----+-----+-----+\n"
             "| z3-9-5-126-127-00000001 | 9  | not activated |"
             " ---- | AIX/Linux | 00000000 | 512 |  2  | 1.5 |\n"
             "+-------------------------+----+---------------+"
             "------+-----------+----------+-----+-----+-----+\n"],
            ["--------------------\n"
             "| LogicalPartition |\n"
             "--------------------\n"
             "  allow_perf_data_collection=False\n"
             "  assoc_sys_uuid=a168a3ec-bb3e-3ead-86c1-7d98b9d50239\n"
             "  avail_priority=127\n"
             "  bootmode=Normal\n"
             "  cur_dram_pmem_volumes=2\n"
             "  cur_pmem_volumes=2\n"
             "  current_secure_boot=1\n"
             "  disable_secure_boot=False\n"
             "  env=AIX/Linux\n"
             "  id=9\n"
             "  is_mgmt_partition=True\n"
             "  is_service_partition=False\n"
             "  keylock_pos=normal\n"
             "  keystore_kbytes=4\n"
             "  kvm_capable=False\n"
             "  kvm_memory_overhead=0.0\n"
             "  max_dram_pmem_volumes=4\n"
             "  max_pmem_volumes=964\n"
             "  migration_state=Not_Migrating\n"
             "  name=z3-9-5-126-127-00000001\n"
             "  operating_system=Linux/Red Hat 2.6.32-358.el6.ppc64 6.4\n"
             "  partition_uuid=089FFB20-5D19-4A8C-BB80-13650627D985\n"
             "  pending_proc_compat_mode=default\n"
             "  pending_secure_boot=2\n"
             "  proc_compat_mode=POWER6_Plus\n"
             "  ref_code=00000000\n"
             "  ref_code_full=time_stamp=08/13/2016 23:52:08,refcode=00000000,"
             "word2=03D00000,fru_call_out_loc_codes=#47-Ubuntu SMP Fri Jun 24 "
             "10:09:20 UTC 2016\n"
             "  restrictedio=False\n"
             "  rmc_state=inactive\n"
             "  srr_enabled=True\n"
             "  state=not activated\n"
             "  uuid=089FFB20-5D19-4A8C-BB80-13650627D985\n"
             "  virtual_serial_number=ABC0001\n"
             "  virtual_software_tier=null\n"
             "  -------------------------\n"
             "  | PartitionCapabilities |\n"
             "  -------------------------\n"
             "    io_dlpar=True\n"
             "    mem_dlpar=False\n"
             "    proc_dlpar=True\n"
             "  ----------------------------\n"
             "  | PartitionIOConfiguration |\n"
             "  ----------------------------\n"
             "    max_virtual_slots=64\n"
             "  --------------------------------\n"
             "  | PartitionMemoryConfiguration |\n"
             "  --------------------------------\n"
             "    ame_enabled=False\n"
             "    current=512\n"
             "    desired=512\n"
             "    exp_factor=0.0\n"
             "    max=512\n"
             "    min=512\n"
             "    shared_enabled=False\n"
             "  -----------------------------------\n"
             "  | PartitionProcessorConfiguration |\n"
             "  -----------------------------------\n"
             "    has_dedicated=False\n"
             "    sharing_mode=uncapped\n"
             "    -----------------------------------\n"
             "    | DedicatedProcessorConfiguration |\n"
             "    -----------------------------------\n"
             "      desired=0\n"
             "      max=0\n"
             "      min=0\n"
             "    --------------------------------\n"
             "    | SharedProcessorConfiguration |\n"
             "    --------------------------------\n"
             "      desired_units=1.5\n"
             "      desired_virtual=2\n"
             "      max_units=2.5\n"
             "      max_virtual=3\n"
             "      min_units=0.5\n"
             "      min_virtual=1\n"
             "      pool_id=9\n"
             "      uncapped_weight=128\n"],
            ["<LogicalPartition:LogicalPartition xmlns:Logical"],
            ['name', 'LogicalPartition.state', 'mem_config', 'rmc_ip'],
            ["name=z3-9-5-126-127-00000001,state=not activated,"
             "ame_enabled=False,current=512,desired=512,exp_factor=0.0,"
             "max=512,min=512,shared_enabled=False,rmc_ip=\n"],
            ["z3-9-5-126-127-00000001,not activated,False,512,512,0.0,512,512,"
             "False,\n"],
            ["name___z3-9-5-126-127-00000001,state___not activated,ame_enabled"
             "___False,current___512,desired___512,exp_factor___0.0,max___512,"
             "min___512,shared_enabled___False,rmc_ip___\n"],
            ["name=z3-9-5-126-127-00000001___state=not activated___ame_enabled"
             "=False___current=512___desired=512___exp_factor=0.0___max=512___"
             "min=512___shared_enabled=False___rmc_ip=\n"],
            ["name=z3-9-5-126-127-00000001,state=not activated,"
             "ame_enabled=False,current=512,desired=512,exp_factor=0.0,"
             "max=512,min=512,shared_enabled=False,rmc_ip=\n"],
            ["name___z3-9-5-126-127-00000001___state___not activated___"
             "ame_enabled___False___current___512___desired___512___"
             "exp_factor___0.0___max___512___min___512___shared_enabled___"
             "False___rmc_ip___\n"],
            'LogicalPartition')

        cp = test_util.setup_mock(self, args=['vm', 'list', '--max-width',
                                              '200'],
                                  adpt_fx=self.adpt,
                                  wrps=lpar_w.LPAR.wrap(self.resp))
        cp.obj_type.execute()
        exp_out = (
            "Logical Partitions\n"
            "+----------------------------------+----+---------------+"
            "----------+-----------+-------------+------+-----+-----+\n"
            "|               Name               | ID |     State     |"
            "   RMC    |    Env    |   Ref Code  | Mem  | CPU | Ent |\n"
            "+----------------------------------+----+---------------+"
            "----------+-----------+-------------+------+-----+-----+\n"
            "|           powervc1_126           | 2  | not activated |"
            " inactive | AIX/Linux |             |  0   |  0  | 0.0 |\n"
            "|              linux1              | 2  | not activated |"
            "   none   | AIX/Linux |   00000000  | None |  0  |     |\n"
            "|      z3-9-5-125-87-00000006      | 2  |    running    |"
            "  active  | AIX/Linux | Linux ppc64 | 512  |  1  | 0.7 |\n"
            "|      z3-9-5-127-7-00000029       | 3  |    running    |"
            "  active  | AIX/Linux |             | 512  |  1  |     |\n"
            "|     z3-9-5-126-208-000001c9      | 3  | not activated |"
            " inactive | AIX/Linux |   00000000  | 2048 |  1  | 0.5 |\n"
            "|      z3-9-5-125-87-00000007      | 4  | not activated |"
            " inactive | AIX/Linux |   00000000  | 768  |  1  | 0.5 |\n"
            "|            test_scsi             | 4  | not activated |"
            " inactive | AIX/Linux |   00000000  | 256  |  1  |     |\n"
            "|     z3-9-5-126-106-00000001      | 5  | not activated |"
            " inactive | AIX/Linux |             |  0   |  0  | 0.0 |\n"
            "|      z3-9-5-127-7-0000002c       | 5  | not activated |"
            " inactive | AIX/Linux |             | 512  |  1  | 0.5 |\n"
            "|      z3-9-5-127-7-0000002e       | 6  |    running    |"
            "  active  | AIX/Linux | Linux ppc64 | 512  |  1  | 0.5 |\n"
            "| brnelson-z3-9-5-126-188-00000051 | 7  | not activated |"
            " inactive | AIX/Linux |             | 512  |  1  | 0.5 |\n"
            "|              kevin1              | 7  | not activated |"
            " inactive | AIX/Linux |             | 256  |  1  | 0.1 |\n"
            "|     z3-9-5-127-104-00000029      | 8  | not activated |"
            " inactive | AIX/Linux |             | 512  |  1  | 0.5 |\n"
            "|     z3-9-5-126-127-00000001      | 9  | not activated |"
            "   ----   | AIX/Linux |   00000000  | 512  |  2  | 1.5 |\n"
            "|     z3-9-5-126-208-000001eb      | 10 | not activated |"
            " inactive | AIX/Linux |   00000000  | 512  |  1  | 0.5 |\n"
            "|     z3-9-5-126-208-000001f0      | 11 | not activated |"
            " inactive | AIX/Linux |   00000000  | 2048 |  1  | 0.5 |\n"
            "|      z3-9-5-125-87-00000005      | 12 |    running    |"
            " inactive | AIX/Linux | Linux ppc64 | 1024 |  1  | 0.6 |\n"
            "|     z3-9-5-126-153-0000002a      | 13 | not activated |"
            " inactive | AIX/Linux |   00000000  |  0   |  0  | 0.0 |\n"
            "|     z3-9-5-126-168-00000002      | 15 | not activated |"
            " inactive | AIX/Linux |   00000000  | 512  |  2  |     |\n"
            "|      z3-9-5-125-63-0000003d      | 16 | not activated |"
            " inactive | AIX/Linux |             | 512  |  1  | 0.5 |\n"
            "|     z3-9-5-126-127-00000015      | 17 | not activated |"
            " inactive | AIX/Linux |   00000000  | 512  |  1  | 0.5 |\n"
            "+----------------------------------+----+---------------+"
            "----------+-----------+-------------+------+-----+-----+\n")
        self.assertEqual(cp.obj_type.cli_out.getvalue(), exp_out)

    @mock.patch('pvmcli.command.object_type.ExecutionContext.get_msys_io_slot')
    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.update')
    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.create')
    def test_create(self, create_mock, update_mock, msys_io_mock):
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-type',
                                              'dedicated', '--proc', '1',
                                              '--sharing-mode', 'normal',
                                              '--type', 'AIX/Linux'],
                                  adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        cp.obj_type.ec.wrappers[0].update.assert_not_called()
        create_mock.reset_mock()

        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-type',
                                              'dedicated', '--proc', '1',
                                              '--sharing-mode', 'normal',
                                              '--type', 'oS400'],
                                  adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        cp.obj_type.ec.wrappers[0].update.assert_not_called()
        create_mock.reset_mock()

        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-type',
                                              'dedicated', '--proc', '1',
                                              '--sharing-mode', 'normal',
                                              '--type', 'aIX/linUx'],
                                  adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        cp.obj_type.ec.wrappers[0].update.assert_not_called()
        create_mock.reset_mock()

        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-unit', '1',
                                              '--proc-type', 'shared',
                                              '--proc', '1', '--sharing-mode',
                                              'capped', '--type',
                                              'AIX/Linux'],
                                  adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        cp.obj_type.ec.wrappers[0].update.assert_not_called()
        create_mock.reset_mock()

        # Test adding ProfileIOSlot
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-type',
                                              'dedicated', '--proc', '1',
                                              '--sharing-mode', 'normal',
                                              '--type', 'AIX/Linux',
                                              '--drc-name', 'io_slot_name'],
                                  adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        # DRC additions called through an update
        cp.obj_type.ec.wrappers[0].update.assert_called_once_with()
        cp.obj_type.ec.wrappers[0].update.reset_mock()
        create_mock.reset_mock()

        # Test adding ProfileIOSlot and an ID
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-type',
                                              'dedicated', '--proc', '1',
                                              '--sharing-mode', 'normal',
                                              '--type', 'AIX/Linux',
                                              '--drc-name', 'io_slot_name',
                                              '--id', '123'],
                                  adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        # DRC additions called through an update
        cp.obj_type.ec.wrappers[0].update.assert_called_once_with()
        cp.obj_type.ec.wrappers[0].update.reset_mock()
        create_mock.reset_mock()

        # Test adding pool id
        cp = test_util.setup_mock(self, args=[
            'LogicalPartition', 'create', '--name', 'lpar_name', '--mem',
            '1024', '--proc-type', 'dedicated', '--proc', '1',
            '--sharing-mode', 'normal', '--type', 'AIX/Linux', '--id',
            '123', '--pool', 'id=1'], adpt_fx=self.adpt)
        cp.obj_type.execute()
        create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
        cp.obj_type.ec.wrappers[0].update.assert_not_called()
        create_mock.reset_mock()

        with mock.patch("pypowervm.wrappers.shared_proc_pool."
                        "SharedProcPool.search"):
            # Test adding pool name
            cp = test_util.setup_mock(self, args=[
                'LogicalPartition', 'create', '--name', 'lpar_name', '--mem',
                '1024', '--proc-type', 'dedicated', '--proc', '1',
                '--sharing-mode', 'normal', '--type', 'AIX/Linux', '--id',
                '123', '--pool', 'name=pool_abc'], adpt_fx=self.adpt)
            cp.obj_type.execute()
            create_mock.assert_called_once_with(parent=cp.obj_type.ec.msys)
            cp.obj_type.ec.wrappers[0].update.assert_not_called()
            create_mock.reset_mock()

        # No name argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--mem',
                                        '1024', '--proc-type', 'shared',
                                        '--proc', '1', '--sharing-mode',
                                        'capped', '--type', 'AIX/Linux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No mem argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--name',
                                        'lpar_name', '--proc-type', 'shared',
                                        '--proc', '1', '--sharing-mode',
                                        'capped', '--type', 'AIX/Linux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No proc type argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--name',
                                        'lpar_name', '--mem', '1',
                                        '--proc', '1', '--sharing-mode',
                                        'capped', '--type', 'AIX/Linux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No proc argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--name',
                                        'lpar_name', '--proc-type', 'shared',
                                        '--mem', '1024', '--sharing-mode',
                                        'capped', '--type', 'AIX/Linux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No sharing-mode argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--name',
                                        'lpar_name', '--proc-type', 'shared',
                                        '--mem', '1024', '--proc', '1',
                                        '--type', 'AIX/Linux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No type argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--name',
                                        'lpar_name', '--proc-type', 'shared',
                                        '--mem', '1024', '--proc', '1',
                                        '--sharing-mode', 'capped'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # Invalid type argument
        cp = cli_parser.CLIParser(args=['LogicalPartition', 'create', '--name',
                                        'lpar_name', '--proc-type', 'shared',
                                        '--mem', '1024', '--proc', '1',
                                        '--sharing-mode', 'capped', '--type',
                                        'AIX/Linediux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # Sharing proc type with dedicated sharing mode
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-unit', '1',
                                              '--proc-type', 'shared',
                                              '--proc', '1', '--sharing-mode',
                                              'normal', '--type',
                                              'AIX/Linux'],
                                  adpt_fx=self.adpt)
        self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
        create_mock.assert_not_called()

        # Dedicated proc type with sharing sharing mode
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'create',
                                              '--name', 'lpar_name', '--mem',
                                              '1024', '--proc-type',
                                              'dedicated', '--proc', '1',
                                              '--sharing-mode', 'capped',
                                              '--type', 'AIX/Linux'],
                                  adpt_fx=self.adpt)
        self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
        create_mock.assert_not_called()

    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.update')
    def test_update(self, ew_update_mock):
        help_strs = UPDATE_USAGE_HELP + SET_FIELD_HELP
        test_util.test_update(
            self, ew_update_mock, 'LogicalPartition',
            ["name=abcd", "PartitionMemoryConfiguration.desired=512"],
            "LogicalPartition.notathing=123", help_strs,
            "Set field LogicalPartition.notathing=123 has an improper format. "
            "Please specify the set field as <Field>=<Value>. Run the command "
            "'pvmctl LogicalPartition update help' for a list of valid "
            "set fields.")

        with mock.patch('pvmcli.command.object_type.update_retry') as ur_mock:
            # Shared Proc Config Tests
            self.assertFalse(self.dwrap.proc_config.has_dedicated)
            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=capped'],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(self.dwrap, {}, cp.obj_type)
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'capped')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=uncapped'],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(self.dwrap, {}, cp.obj_type)
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'uncapped')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=cap'],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(self.dwrap, {}, cp.obj_type)
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'capped')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=uncap'],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(self.dwrap, {}, cp.obj_type)
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'uncapped')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=notathing'],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
            ur_mock.assert_not_called()
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'uncapped')

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=active'],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
            ur_mock.assert_not_called()
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'uncapped')

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=share_idle_"
                                                  "procs"],
                                      adpt_fx=self.adpt, wrps=[self.dwrap])
            self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
            ur_mock.assert_not_called()
            self.assertEqual(self.dwrap.proc_config.sharing_mode, 'uncapped')

            # Dedicated Proc Config
            ded_wrp = lpar_w.LPAR.wrap(self.resp)[4]
            self.assertTrue(ded_wrp.proc_config.has_dedicated)

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=normal'],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle proces')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=share_idle_"
                                                  "procs"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle proces')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=keep"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'keep idle procs')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=keep_idle_"
                                                  "procs"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'keep idle procs')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=active"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle procs active')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=share_idle_"
                                                  "procs_active"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle procs active')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=always"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle procs always')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  "sharing_mode=share_idle_"
                                                  "procs_always"],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            cp.obj_type.execute()
            ur_mock.assert_called_once_with(ded_wrp, {}, cp.obj_type)
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle procs always')
            ur_mock.reset_mock()

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=notathing'],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
            ur_mock.assert_not_called()
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle procs always')

            cp = test_util.setup_mock(self, args=['vm', 'update', '-s',
                                                  'sharing_mode=cap'],
                                      adpt_fx=self.adpt, wrps=[ded_wrp])
            self.assertRaises(exc.InvalidSharingModeError, cp.obj_type.execute)
            ur_mock.assert_not_called()
            self.assertEqual(ded_wrp.proc_config.sharing_mode,
                             'sre idle procs always')

    @mock.patch('pvmcli.command.base_partition.BasePartition._validate_attr',
                return_value=0)
    @mock.patch("pypowervm.tasks.power.power_on")
    def test_power_on(self, power_on_mock, validate_lpar_mock):
        test_util.test_power_on(self, power_on_mock, test_util.LPAR_OBJ_NAME)
        for b_opt in lpar_w.IPLSrc.ALL_VALUES:
            cp = test_util.setup_mock(self, ['lpar', 'power-on',
                                             '--iplsource', b_opt],
                                      self.adpt, [self.dwrap])
            cp.obj_type.execute()
            power_on_mock.assert_called_once_with(
                self.dwrap, cp.obj_type.ec.msys.uuid,
                add_parms={lpar_w.IPLSrc.KEY: b_opt})
            power_on_mock.reset_mock()

            cp = test_util.setup_mock(self, ['lpar', 'power-on',
                                             '--iplsource', b_opt,
                                             '--bootstring', 'abcd'],
                                      self.adpt, [self.dwrap])
            cp.obj_type.execute()
            power_on_mock.assert_called_once_with(
                self.dwrap, cp.obj_type.ec.msys.uuid,
                add_parms={lpar_w.IPLSrc.KEY: b_opt, 'bootstring': 'abcd'})
            power_on_mock.reset_mock()

    @mock.patch('pvmcli.command.base_partition.BasePartition._validate_attr',
                return_value=0)
    @mock.patch("pvmcli.util.util.input", return_value="Y")
    @mock.patch("pypowervm.tasks.power.PowerOp.stop")
    def test_power_off_restart(self, power_off_mock,
                               input_mock, validate_mock):
        test_util.test_power_off(self, power_off_mock=power_off_mock,
                                 obj_name=test_util.LPAR_OBJ_NAME)
        p_opt_o = 'pypowervm.tasks.power_opts.PowerOffOpts.operation'
        p_opt_i = 'pypowervm.tasks.power_opts.PowerOffOpts.immediate'
        p_opt_r = 'pypowervm.tasks.power_opts.PowerOffOpts.restart'
        with mock.patch(p_opt_o) as ptc_o, mock.patch(p_opt_i) as ptc_i,\
                mock.patch(p_opt_r) as ptc_r:
            cp = test_util.setup_mock(self, ['lpar', 'dumprestart'],
                                      self.adpt, [self.dwrap])
            cp.obj_type.execute()
            ptc_o.assert_called()
            ptc_i.assert_not_called()
            ptc_r.assert_not_called()
            power_off_mock.assert_called()
            test_util.validate_input_called(test_util.LPAR_OBJ_NAME,
                                            input_mock, 'dumprestart')
            ptc_o.reset_mock()
            power_off_mock.reset_mock()

    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.delete')
    def test_delete(self, ew_mock_delete):
        ''' Test delete operation on LogicalPartition ObjectType. '''
        args = ['LogicalPartition', 'delete', '-i', 'uuid=abcd']
        cp = test_util.setup_mock(self, args, self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        ew_mock_delete.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                               reusevsn=False)
        ew_mock_delete.reset_mock()

    @mock.patch('pypowervm.tasks.migration.migrate_lpar')
    def test_migrate(self, mig_mock):
        ''' Test migrate operation on LogicalPartition ObjectType. '''
        # Migrate
        # Minimum args
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'migrate',
                                              '--mgmt-svr', '127.0.0.1',
                                              '-i', 'name=foo'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        mig_mock.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                         timeout=0,
                                         validate_only=False,
                                         tgt_mgmt_svr='127.0.0.1',
                                         tgt_mgd_sys=None, tgt_mgmt_usr=None,
                                         sdn_override=False,
                                         spp_id=None,
                                         pmem_volumes=None,
                                         message_list=[])
        self.assertEqual(cp.obj_type.cli_out.getvalue(),
                         "Migration of logical partition z3-9-5-126-127-"
                         "00000001 starting. This may take a while.\n"
                         "Migration of logical partition z3-9-5-126-127-"
                         "00000001 succeeded.\n")
        mig_mock.reset_mock()

        # No args
        self.assertRaises(SystemExit, test_util.setup_mock,
                          self, ['LogicalPartition', 'migrate'], self.adpt,
                          wrps=[self.dwrap])
        mig_mock.assert_not_called()

        # All args
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'migrate',
                                              '--mgmt-svr', '127.0.0.1',
                                              '--sys', 'sysarg', '--validate',
                                              '--mgmt-usr', 'user',
                                              '--timeout', '2', '-i',
                                              'name=foo', '--allow-ovs',
                                              '--spp-id', '1',
                                              '--pmem-volumes',
                                              'Src1/Tgt1,Src2/Tgt2'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        mig_mock.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                         timeout=2,
                                         validate_only=True,
                                         tgt_mgmt_svr='127.0.0.1',
                                         tgt_mgd_sys='sysarg',
                                         tgt_mgmt_usr='user',
                                         sdn_override=True,
                                         spp_id=1,
                                         pmem_volumes='Src1/Tgt1,Src2/Tgt2',
                                         message_list=[])
        self.assertEqual(cp.obj_type.cli_out.getvalue(),
                         "Validating logical partition z3-9-5-126-127-0000"
                         "0001 migration...\nValidation of logical partition "
                         "z3-9-5-126-127-00000001 succeeded.\n")
        mig_mock.reset_mock()

        mig_mock.side_effect = Exception()
        cp = test_util.setup_mock(self, args=['LogicalPartition', 'migrate',
                                              '--mgmt-svr', '127.0.0.1',
                                              '--sys', 'sysarg', '--validate',
                                              '--mgmt-usr', 'user',
                                              '--timeout', '2', '-i',
                                              'name=foo'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        self.assertRaises(Exception, cp.obj_type.execute)
        self.assertEqual(
            cp.obj_type.cli_err.getvalue(),
            "Validation of logical partition z3-9-5-126-127-00000001 "
            "failed.\n")

        # Failure cases
        # -i required
        self.assertRaises(SystemExit, test_util.setup_mock, self,
                          args=['LogicalPartition', 'migrate',
                                '--mgmt-svr', '127.0.0.1',
                                '--sys', 'sysarg', '--validate',
                                '--mgmt-usr', 'user', '--timeout', '2'],
                          adpt_fx=self.adpt, wrps=[self.dwrap])
        mig_mock.assert_not_called()

    @mock.patch('pypowervm.tasks.migration.migrate_recover')
    def test_migrate_recover(self, rec_mock):
        ''' Test migrate-recover operation on LogicalPartition ObjectType. '''
        # Minimum args
        cp = test_util.setup_mock(self, args=['LogicalPartition',
                                              'migrate-recover', '-i',
                                              'name=foo'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        rec_mock.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                         timeout=0)
        rec_mock.reset_mock()

        # All args
        cp = test_util.setup_mock(self, args=['LogicalPartition',
                                              'migrate-recover', '-i',
                                              'name=foo', '--force',
                                              '--timeout', '2'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        rec_mock.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                         timeout=2, force=True)

        # Failure cases
        # -i required
        self.assertRaises(SystemExit, test_util.setup_mock, self,
                          args=['LogicalPartition', 'migrate-recover',
                                '--force', '--timeout', '2'],
                          adpt_fx=self.adpt, wrps=[self.dwrap])
        rec_mock.assert_not_called()

    @mock.patch('pypowervm.tasks.migration.migrate_abort')
    def test_migrate_stop(self, abort_mock):
        ''' Test migrate-stop operation on LogicalPartition ObjectType. '''
        # Minimum args
        cp = test_util.setup_mock(self, args=['LogicalPartition',
                                              'migrate-stop', '-i',
                                              'name=foo'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        abort_mock.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                           timeout=0)
        abort_mock.reset_mock()

        # All args
        cp = test_util.setup_mock(self, args=['LogicalPartition',
                                              'migrate-stop', '-i',
                                              'name=foo', '--timeout', '2'],
                                  adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        abort_mock.assert_called_once_with(cp.obj_type.ec.wrappers[0],
                                           timeout=2)

        # Failure cases
        # -i required
        self.assertRaises(SystemExit, test_util.setup_mock, self,
                          args=['LogicalPartition', 'migrate-stop',
                                '--timeout', '2'],
                          adpt_fx=self.adpt, wrps=[self.dwrap])
        abort_mock.assert_not_called()

    @mock.patch('pypowervm.tasks.ibmi.start_panel_job')
    def test_ibmi_panel(self, panel_mock):
        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'dumprestart'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'dumprestart',
                                           timeout=1800)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op', 'dston',
                        '--timeout', '500'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'dston', timeout=500)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'retrydump', '--timeout', '600'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'retrydump',
                                           timeout=600)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'remotedstoff', '--timeout', '777'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'remotedstoff',
                                           timeout=777)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'remotedston', '--timeout', '1'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'remotedston',
                                           timeout=1)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'iopreset'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'iopreset',
                                           timeout=1800)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op', 'iopdump',
                        '--timeout', '500'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'iopdump',
                                           timeout=500)
        panel_mock.reset_mock()

        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'consoleservice', '--timeout', '500'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        panel_mock.assert_called_once_with(self.dwrap, 'consoleservice',
                                           timeout=500)
        panel_mock.reset_mock()

    def test_ibmi_panel_exceptions(self):
        cp = test_util.setup_mock(
            self, args=['lpar', 'panel', '-i', 'name=foo', '--op',
                        'dumprestart', '--timeout', '500'],
            adpt_fx=self.adpt, wrps=[None])
        self.assertRaises(
            pvm_exc.PanelFunctionRequiresPartition, cp.obj_type.execute)

        # argparse choices will force failure on --op processing
        self.assertRaises(SystemExit, test_util.setup_mock, self, args=[
            'lpar', 'panel', '-i', 'name=foo', '--op', 'notright',
            '--timeout', '500'], adpt_fx=self.adpt, wrps=[None])
