# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017.
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock

import pvmcli.command.managed_system as ms_c
import pvmcli.help.help_managed_system as help_msys
import pvmcli.tests.util.test_util as test_util
import pvmcli.util.exceptions as exc
import pvmcli.util.output_processor as out_proc

import pypowervm.tests.test_utils.test_wrapper_abc as twa
import pypowervm.wrappers.iocard as iocard
import pypowervm.wrappers.managed_system as ms_w
import pypowervm.wrappers.mtms as mtms


LIST_USAGE_HELP = (["usage: pvmctl [help | --help] ManagedSystem list {args}"])


DISPLAY_FIELD_HELP = (
    ["    AssociatedSystemCapabilities.active_lpar_mobility_capable\n"
     "    AssociatedSystemCapabilities.active_memory_expansion_capable\n"
     "    AssociatedSystemCapabilities.custom_mac_addr_capable\n"
     "    AssociatedSystemCapabilities.dedicated_processor_partition_capable\n"
     "    AssociatedSystemCapabilities.disable_secure_boot_capable\n"
     "    AssociatedSystemCapabilities.ibmi_lpar_mobility_capable\n"
     "    AssociatedSystemCapabilities.ibmi_nativeio_capable\n"
     "    AssociatedSystemCapabilities.ibmi_network_install_capable\n"
     "    AssociatedSystemCapabilities.ibmi_network_install_iscsi_capable\n"
     "    AssociatedSystemCapabilities.ibmi_restrictedio_capable\n"
     "    AssociatedSystemCapabilities.ibmi_virtual_software_tier_capable\n"
     "    AssociatedSystemCapabilities.inactive_lpar_mobility_capable\n"
     "    AssociatedSystemCapabilities.ioslot_owner_assignment_capable\n"
     "    AssociatedSystemCapabilities.kvm_capable\n"
     "    AssociatedSystemCapabilities.partition_key_store_capable\n"
     "    AssociatedSystemCapabilities.partition_secure_boot_capable\n"
     "    AssociatedSystemCapabilities.persistent_memory_capable\n"
     "    AssociatedSystemCapabilities.physical_page_table_ratio_capable\n"
     "    AssociatedSystemCapabilities.shared_processor_pool_capable\n"
     "    AssociatedSystemCapabilities.simplified_remote_restart_capable\n"
     "    AssociatedSystemCapabilities.virtual_serial_number_capable\n"
     "    AssociatedSystemCapabilities.vnic_capable\n"
     "    AssociatedSystemCapabilities.vnic_failover_capable\n"
     "    AssociatedSystemIOConfiguration.avail_wwpns\n"
     "    AssociatedSystemIOConfiguration.io_slots\n"
     "    AssociatedSystemIOConfiguration.sriov_adapters\n"
     "    AssociatedSystemIOConfiguration.wwpn_prefix\n"
     "    IBMiVirtualSoftwareTier.maximum_allowed_memory\n"
     "    IBMiVirtualSoftwareTier.maximum_allowed_processors\n"
     "    IBMiVirtualSoftwareTier.tier_name\n"
     "    IOSlot.bus_grp_required\n"
     "    IOSlot.description\n"
     "    IOSlot.drc_index\n"
     "    IOSlot.drc_name\n"
     "    IOSlot.feat_codes\n"
     "    IOSlot.part_id\n"
     "    IOSlot.part_name\n"
     "    IOSlot.part_type\n"
     "    IOSlot.part_uuid\n"
     "    IOSlot.pci_class\n"
     "    IOSlot.pci_dev_id\n"
     "    IOSlot.pci_rev_id\n"
     "    IOSlot.pci_subsys_dev_id\n"
     "    IOSlot.pci_subsys_vendor_id\n"
     "    IOSlot.pci_vendor_id\n"
     "    MachineTypeModelAndSerialNumber.machine_type\n"
     "    MachineTypeModelAndSerialNumber.model\n"
     "    MachineTypeModelAndSerialNumber.mtms_str\n"
     "    MachineTypeModelAndSerialNumber.serial\n"
     "    ManagedSystem.asio_config\n"
     "    ManagedSystem.associated_capabilities\n"
     "    ManagedSystem.cur_pmem_volumes\n"
     "    ManagedSystem.default_ppt_ratio\n"
     "    ManagedSystem.dram_pmem_vol_blksize\n"
     "    ManagedSystem.dram_pmem_vol_cursize\n"
     "    ManagedSystem.dram_pmem_vol_size\n"
     "    ManagedSystem.firmware_memory\n"
     "    ManagedSystem.host_ip_address\n"
     "    ManagedSystem.ibmivirtualsoftwaretiers\n"
     "    ManagedSystem.lpar_keystore_max_kbytes\n"
     "    ManagedSystem.lpar_keystore_min_kbytes\n"
     "    ManagedSystem.max_aix_lnx_pmem_volumes\n"
     "    ManagedSystem.max_os400_pmem_volumes\n"
     "    ManagedSystem.max_pmem_volumes\n"
     "    ManagedSystem.max_procs_per_aix_linux_lpar\n"
     "    ManagedSystem.max_sys_procs_limit\n"
     "    ManagedSystem.max_sys_vcpus_limit\n"
     "    ManagedSystem.max_vcpus_per_aix_linux_lpar\n"
     "    ManagedSystem.max_vios_pmem_volumes\n"
     "    ManagedSystem.memory_configurable\n"
     "    ManagedSystem.memory_free\n"
     "    ManagedSystem.memory_region_size\n"
     "    ManagedSystem.memory_total\n"
     "    ManagedSystem.metered_pool_id\n"
     "    ManagedSystem.migration_data\n"
     "    ManagedSystem.min_proc_units\n"
     "    ManagedSystem.mtms\n"
     "    ManagedSystem.page_table_ratio\n"
     "    ManagedSystem.proc_compat_modes\n"
     "    ManagedSystem.proc_units\n"
     "    ManagedSystem.proc_units_avail\n"
     "    ManagedSystem.proc_units_configurable\n"
     "    ManagedSystem.processor_is_throttled\n"
     "    ManagedSystem.session_is_controller\n"
     "    ManagedSystem.session_is_master\n"
     "    ManagedSystem.supported_ibmi_console_codepage\n"
     "    ManagedSystem.supported_pmem_vol_devtypes\n"
     "    ManagedSystem.system_name\n"
     "    ManagedSystem.system_state\n"
     "    ManagedSystem.uuid\n"
     "    ManagedSystem.vios_links\n"
     "    PhysicalPort.allocated_capacity\n"
     "    PhysicalPort.cfg_lps\n"
     "    PhysicalPort.cfg_max_lps\n"
     "    PhysicalPort.curr_speed\n"
     "    PhysicalPort.flow_ctl\n"
     "    PhysicalPort.label\n"
     "    PhysicalPort.link_status\n"
     "    PhysicalPort.loc_code\n"
     "    PhysicalPort.min_granularity\n"
     "    PhysicalPort.mtu\n"
     "    PhysicalPort.port_id\n"
     "    PhysicalPort.sriov_adap_id\n"
     "    PhysicalPort.sublabel\n"
     "    PhysicalPort.supp_max_lps\n"
     "    PhysicalPort.switch_mode\n"
     "    SRIOVAdapter.description\n"
     "    SRIOVAdapter.dev_name\n"
     "    SRIOVAdapter.dev_type\n"
     "    SRIOVAdapter.drc_name\n"
     "    SRIOVAdapter.id\n"
     "    SRIOVAdapter.mode\n"
     "    SRIOVAdapter.personality\n"
     "    SRIOVAdapter.phys_loc_code\n"
     "    SRIOVAdapter.phys_ports\n"
     "    SRIOVAdapter.sriov_adap_id\n"
     "    SRIOVAdapter.state\n"
     "    SRIOVAdapter.udid\n"])


class ManagedSystemTest(twa.TestWrapper):
    ''' Test framework for ManagedSystem Object Type. '''
    @property
    def file(self):
        return 'sys_with_sriov.txt'

    @property
    def wrapper_class_to_test(self):
        return ms_w.System

    def test_managed_system(self):
        obj_type = ms_c.ManagedSystem()
        # Test Object Type data
        test_util.test_obj_type(self, obj_type, 'ManagedSystem', ms_w.System,
                                field_models=[
                                    ms_w.System, mtms.MTMS,
                                    ms_w.ASIOConfig, ms_w.IOSlot,
                                    iocard.SRIOVAdapter,
                                    iocard.SRIOVConvPPort,
                                    iocard.SRIOVEthPPort,
                                    iocard.SRIOVRoCEPPort,
                                    ms_w.AssociatedSystemCapabilities,
                                    ms_w.VirtualSoftwareTiers,
                                    ms_w.IBMiVirtualSoftwareTier],
                                name_to_ops={'list',
                                             'request-master',
                                             'request-controller',
                                             'list-public-key',
                                             'add-auth-key',
                                             'list-auth-keys',
                                             'rest-log-levels'},
                                exp_help=help_msys.help_text)
        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={
                obj_type.name_to_ops['list']: None,
                obj_type.name_to_ops['request-master']: None,
                obj_type.name_to_ops['request-controller']: None},
            child_model_ops={obj_type.name_to_ops['list']: None,
                             obj_type.name_to_ops['request-master']: None,
                             obj_type.name_to_ops['request-controller']: None},
            elem_wrp_func_ops={
                obj_type.name_to_ops['list']: None,
                obj_type.name_to_ops['request-master']: None,
                obj_type.name_to_ops['request-controller']: None})

    def test_overrides(self):
        ''' Test the overridden Field objects for ManagedSystem ObjectType '''
        msys = ms_c.ManagedSystem()
        a_config_df = msys.name_to_ops['list'].get_disp_field(ms_w.System,
                                                              'asio_config')
        mtms_df = msys.name_to_ops['list'].get_disp_field(ms_w.System, 'mtms')
        io_slots_df = msys.name_to_ops['list'].get_disp_field(ms_w.ASIOConfig,
                                                              'io_slots')
        drc_df = (msys.name_to_ops['list']
                  .get_disp_field(ms_w.IOSlot, 'drc_index'))
        ivsts_df = (msys.name_to_ops['list']
                    .get_disp_field(ms_w.System, 'ibmivirtualsoftwaretiers'))
        ivst_df = (msys.name_to_ops['list']
                   .get_disp_field(ms_w.VirtualSoftwareTiers,
                                   'virtual_software_tier'))

        self.assertEquals(a_config_df.label, ms_w.ASIOConfig.schema_type)
        self.assertEquals(mtms_df.label, mtms.MTMS.schema_type)
        self.assertEquals(io_slots_df.label, ms_w.IOSlot.schema_type)
        self.assertEquals(drc_df.out_func, out_proc.hex_no_prefix)
        self.assertEquals(ivsts_df.label,
                          ms_w.VirtualSoftwareTiers.schema_type)
        self.assertEquals(ivst_df.label,
                          ms_w.IBMiVirtualSoftwareTier.schema_type)

    def test_list(self):
        test_util.test_list(
            self, 'ManagedSystem',
            LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
            ["Managed System\n"
             "+----------------------------+------------------+--------+"
             "----------+------+-----------+\n"
             "|            Name            |       MTMS       |  Mem   |"
             " Free Mem | Proc | Free Proc |\n"
             "+----------------------------+------------------+--------+"
             "----------+------+-----------+\n"
             "| z1403ce-8408-E8E-SN10678CV | 8408-E8E*10678CV | 524288 |"
             "  491008  | 40.0 |    36.0   |\n"
             "+----------------------------+------------------+--------+"
             "----------+------+-----------+\n"],
            ["-----------------\n"
             "| ManagedSystem |\n"
             "-----------------\n"
             "  default_ppt_ratio=6\n"
             "  firmware_memory=8192\n"
             "  host_ip_address=9.5.249.35\n"
             "  lpar_keystore_max_kbytes=0\n"
             "  lpar_keystore_min_kbytes=0\n"
             "  max_procs_per_aix_linux_lpar=64\n"
             "  max_sys_procs_limit=256\n"
             "  max_sys_vcpus_limit=256\n"
             "  max_vcpus_per_aix_linux_lpar=64\n"
             "  memory_configurable (MB)=524288\n"
             "  memory_free (MB)=491008\n"
             "  memory_region_size (MB)=256\n"
             "  memory_total (MB)=524288\n"
             "  min_proc_units=0.05\n"
             "  page_table_ratio=6\n"
             "  proc_compat_modes=('default', 'POWER6', 'POWER6_Plus', "
             "'POWER7', 'POWER8')\n"
             "  proc_units=40.0\n"
             "  proc_units_avail=36.0\n"
             "  proc_units_configurable=40.0\n"
             "  processor_is_throttled=False\n"
             "  session_is_controller=True\n"
             "  session_is_master=True\n"
             "  system_name=z1403ce-8408-E8E-SN10678CV\n"
             "  system_state=operating\n"
             "  uuid=95329c84-9a01-330f-809e-2e5577110d1e\n"
             "  -----------------------------------\n"
             "  | AssociatedSystemIOConfiguration |\n"
             "  -----------------------------------\n"
             "    avail_wwpns=0\n"
             "    wwpn_prefix=13857705831041335296\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=SAS RAID Controller, PCIe2, Dual-port 6Gb\n"
             "      drc_index=21010021\n"
             "      drc_name=U78C7.001.RCH0004-P3-R1\n"
             "      pci_class=0104\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=2\n"
             "      pci_subsys_dev_id=04C8\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=034A\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=PCIe2 4-port (10Gb FCoE & 1GbE) SR&RJ45 "
             "Adapter\n"
             "      drc_index=21010020\n"
             "      drc_name=U78C7.001.RCH0004-P1-C8\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=16\n"
             "      pci_subsys_dev_id=040F\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=E220\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=PCIe2 4-port (10Gb FCoE & 1GbE) SR&RJ45 "
             "Adapter\n"
             "      drc_index=21010023\n"
             "      drc_name=U78C7.001.RCH0004-P1-C9\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=16\n"
             "      pci_subsys_dev_id=040F\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=E220\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=Empty slot\n"
             "      drc_index=21010040\n"
             "      drc_name=U78C7.001.RCH0004-P1-C1\n"
             "      feat_codes=0\n"
             "      pci_class=FFFF\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=255\n"
             "      pci_subsys_dev_id=FFFF\n"
             "      pci_subsys_vendor_id=FFFF\n"
             "      pci_vendor_id=FFFF\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=1 Gigabit Ethernet (UTP) 4 Port Adapter "
             "PCIE-4x/Short\n"
             "      drc_index=21010028\n"
             "      drc_name=U78C7.001.RCH0004-P1-C9\n"
             "      feat_codes=5899\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=1\n"
             "      pci_subsys_dev_id=0420\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=1657\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=Empty slot\n"
             "      drc_index=21010048\n"
             "      drc_name=U78C7.001.RCH0004-P1-C2\n"
             "      feat_codes=0\n"
             "      pci_class=FFFF\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=255\n"
             "      pci_subsys_dev_id=FFFF\n"
             "      pci_subsys_vendor_id=FFFF\n"
             "      pci_vendor_id=FFFF\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=Universal Serial Bus UHC Spec\n"
             "      drc_index=2101002b\n"
             "      drc_name=U78C7.001.RCH0004-P1-T1\n"
             "      pci_class=0C03\n"
             "      pci_dev_id=1234\n"
             "      pci_rev_id=2\n"
             "      pci_subsys_dev_id=04B2\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=8241\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=Empty slot\n"
             "      drc_index=2102002c\n"
             "      drc_name=U78C7.001.RCH0004-P1-C6\n"
             "      feat_codes=0\n"
             "      pci_class=FFFF\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=255\n"
             "      pci_subsys_dev_id=FFFF\n"
             "      pci_subsys_vendor_id=FFFF\n"
             "      pci_vendor_id=FFFF\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=2-Port 10GbE RoCE SFP+ Adapter\n"
             "      drc_index=2103002d\n"
             "      drc_name=U78C7.001.RCH0004-P1-C7\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=176\n"
             "      pci_subsys_dev_id=0416\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=6750\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=SAS RAID Controller, PCIe2, Dual-port 6Gb\n"
             "      drc_index=21010011\n"
             "      drc_name=U78C7.001.RCH0004-P3-R2\n"
             "      pci_class=0104\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=2\n"
             "      pci_subsys_dev_id=04C8\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=034A\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=Empty slot\n"
             "      drc_index=21010010\n"
             "      drc_name=U78C7.001.RCH0004-P1-C10\n"
             "      feat_codes=0\n"
             "      pci_class=FFFF\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=255\n"
             "      pci_subsys_dev_id=FFFF\n"
             "      pci_subsys_vendor_id=FFFF\n"
             "      pci_vendor_id=FFFF\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=2-Port 10GbE RoCE SFP+ Adapter\n"
             "      drc_index=21010030\n"
             "      drc_name=U78C7.001.RCH0004-P1-C3\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=176\n"
             "      pci_subsys_dev_id=0416\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=6750\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=Empty slot\n"
             "      drc_index=21010019\n"
             "      drc_name=U78C7.001.RCH0004-P1-C12\n"
             "      feat_codes=0\n"
             "      pci_class=FFFF\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=255\n"
             "      pci_subsys_dev_id=FFFF\n"
             "      pci_subsys_vendor_id=FFFF\n"
             "      pci_vendor_id=FFFF\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=1 Gigabit Ethernet (UTP) 4 Port Adapter "
             "PCIE-4x/Short\n"
             "      drc_index=21010018\n"
             "      drc_name=U78C7.001.RCH0004-P1-C11\n"
             "      feat_codes=5899\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1235\n"
             "      pci_rev_id=1\n"
             "      pci_subsys_dev_id=0420\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=1657\n"
             "    ----------\n"
             "    | IOSlot |\n"
             "    ----------\n"
             "      bus_grp_required=False\n"
             "      description=PCIe2 2-Port 10GbE Base-T Adapter\n"
             "      drc_index=21010038\n"
             "      drc_name=U78C7.001.RCH0004-P1-C4\n"
             "      pci_class=0200\n"
             "      pci_dev_id=1236\n"
             "      pci_rev_id=16\n"
             "      pci_subsys_dev_id=0492\n"
             "      pci_subsys_vendor_id=1014\n"
             "      pci_vendor_id=168E\n"
             "    ----------------\n"
             "    | SRIOVAdapter |\n"
             "    ----------------\n"
             "      description=PCIe2 4-port (10Gb FCoE & 1GbE) SR&RJ45 "
             "Adapter\n"
             "      id=553713696\n"
             "      mode=Sriov\n"
             "      personality=MIN_CAPACITY\n"
             "      phys_loc_code=U78C7.001.RCH0004-P1-C8\n"
             "      sriov_adap_id=1\n"
             "      state=Running\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=2\n"
             "        cfg_max_lps=20\n"
             "        curr_speed=E10Mbps\n"
             "        flow_ctl=False\n"
             "        link_status=True\n"
             "        loc_code=U78C7.001.RCH0004-P1-C8-T1\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=0\n"
             "        sriov_adap_id=1\n"
             "        supp_max_lps=20\n"
             "        switch_mode=Veb\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=0\n"
             "        cfg_max_lps=20\n"
             "        curr_speed=Unknown\n"
             "        flow_ctl=False\n"
             "        link_status=False\n"
             "        loc_code=U78C7.001.RCH0004-P1-C8-T2\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=1\n"
             "        sriov_adap_id=1\n"
             "        supp_max_lps=20\n"
             "        switch_mode=Veb\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=1\n"
             "        cfg_max_lps=4\n"
             "        curr_speed=E1Gbps\n"
             "        flow_ctl=False\n"
             "        link_status=True\n"
             "        loc_code=U78C7.001.RCH0004-P1-C8-T3\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=2\n"
             "        sriov_adap_id=1\n"
             "        supp_max_lps=4\n"
             "        switch_mode=Veb\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=0\n"
             "        cfg_max_lps=4\n"
             "        curr_speed=Unknown\n"
             "        flow_ctl=False\n"
             "        link_status=False\n"
             "        loc_code=U78C7.001.RCH0004-P1-C8-T4\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=3\n"
             "        sriov_adap_id=1\n"
             "        supp_max_lps=4\n"
             "        switch_mode=Veb\n"
             "    ----------------\n"
             "    | SRIOVAdapter |\n"
             "    ----------------\n"
             "      description=PCIe2 4-port (10Gb FCoE & 1GbE) SR&RJ45 "
             "Adapter\n"
             "      id=553713699\n"
             "      mode=Sriov\n"
             "      personality=MAX_MIN_CAPACITY\n"
             "      phys_loc_code=U78C7.001.RCH0004-P1-C9\n"
             "      sriov_adap_id=2\n"
             "      state=Running\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=2\n"
             "        cfg_max_lps=20\n"
             "        curr_speed=E10Mbps\n"
             "        flow_ctl=False\n"
             "        link_status=True\n"
             "        loc_code=U78C7.001.RCH0004-P1-C9-T1\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=0\n"
             "        sriov_adap_id=2\n"
             "        supp_max_lps=20\n"
             "        switch_mode=Veb\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=0\n"
             "        cfg_max_lps=20\n"
             "        curr_speed=Unknown\n"
             "        flow_ctl=False\n"
             "        link_status=False\n"
             "        loc_code=U78C7.001.RCH0004-P1-C9-T2\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=1\n"
             "        sriov_adap_id=2\n"
             "        supp_max_lps=20\n"
             "        switch_mode=Veb\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=1\n"
             "        cfg_max_lps=4\n"
             "        curr_speed=E1Gbps\n"
             "        flow_ctl=False\n"
             "        link_status=True\n"
             "        loc_code=U78C7.001.RCH0004-P1-C9-T3\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=2\n"
             "        sriov_adap_id=2\n"
             "        supp_max_lps=4\n"
             "        switch_mode=Veb\n"
             "      ----------------\n"
             "      | PhysicalPort |\n"
             "      ----------------\n"
             "        allocated_capacity=0.02\n"
             "        cfg_lps=0\n"
             "        cfg_max_lps=4\n"
             "        curr_speed=Unknown\n"
             "        flow_ctl=False\n"
             "        link_status=False\n"
             "        loc_code=U78C7.001.RCH0004-P1-C9-T4\n"
             "        min_granularity=0.02\n"
             "        mtu=E_1500\n"
             "        port_id=3\n"
             "        sriov_adap_id=2\n"
             "        supp_max_lps=4\n"
             "        switch_mode=Veb\n"
             "    ----------------\n"
             "    | SRIOVAdapter |\n"
             "    ----------------\n"
             "      description=PCIe2 4-port (10Gb FCoE & 1GbE) SR&RJ45 "
             "Adapter\n"
             "      id=553713680\n"
             "      mode=Dedicated\n"
             "      phys_loc_code=U78CB.001.WZS06RG-P1-C7\n"
             "      state=NotConfigured\n"
             "    ----------------\n"
             "    | SRIOVAdapter |\n"
             "    ----------------\n"
             "      description=PCIe2 4-port (10Gb FCoE & 1GbE) SR&RJ45 "
             "Adapter\n"
             "      id=553713697\n"
             "      mode=Dedicated\n"
             "      phys_loc_code=U78CB.001.WZS06RG-P1-C5\n"
             "      state=NotConfigured\n"
             "  --------------------------------\n"
             "  | AssociatedSystemCapabilities |\n"
             "  --------------------------------\n"
             "    active_lpar_mobility_capable=True\n"
             "    active_memory_expansion_capable=True\n"
             "    custom_mac_addr_capable=False\n"
             "    dedicated_processor_partition_capable=False\n"
             "    disable_secure_boot_capable=False\n"
             "    ibmi_lpar_mobility_capable=True\n"
             "    ibmi_nativeio_capable=False\n"
             "    ibmi_network_install_capable=False\n"
             "    ibmi_network_install_iscsi_capable=False\n"
             "    ibmi_restrictedio_capable=True\n"
             "    ibmi_virtual_software_tier_capable=False\n"
             "    inactive_lpar_mobility_capable=True\n"
             "    ioslot_owner_assignment_capable=False\n"
             "    kvm_capable=False\n"
             "    partition_key_store_capable=False\n"
             "    partition_secure_boot_capable=False\n"
             "    persistent_memory_capable=False\n"
             "    physical_page_table_ratio_capable=False\n"
             "    shared_processor_pool_capable=True\n"
             "    simplified_remote_restart_capable=True\n"
             "    virtual_serial_number_capable=False\n"
             "    vnic_capable=False\n"
             "    vnic_failover_capable=False\n"
             "  -----------------------------------\n"
             "  | MachineTypeModelAndSerialNumber |\n"
             "  -----------------------------------\n"
             "    machine_type=8408\n"
             "    model=E8E\n"
             "    mtms_str=8408-E8E*10678CV\n"
             "    serial=10678CV\n"],
            ["<ManagedSystem:ManagedSystem xmlns:ManagedSys"],
            ['name', 'number.model', 'ioslot.pci_class', 'port_id'],
            ["system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0C03,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0104,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=FFFF,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=0\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=1\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=2\n"
             "system_name=z1403ce-8408-E8E-SN10678CV,model=E8E"
             ",pci_class=0200,port_id=3\n"],
            ["z1403ce-8408-E8E-SN10678CV,E8E,0104,0\n"],
            ["system_name___z1403ce-8408-E8E-SN10678CV,model___E8E,"
             "pci_class___0104,port_id___0\n"],
            ["system_name=z1403ce-8408-E8E-SN10678CV___model=E8E"
             "___pci_class=0104___port_id=0\n"],
            ["system_name=z1403ce-8408-E8E-SN10678CV,model=E8E,"
             "pci_class=0104,port_id=0___"],
            ["system_name___z1403ce-8408-E8E-SN10678CV___model___"
             "E8E___pci_class___0104___port_id___0___"],
            'ManagedSystem')

    @mock.patch('pypowervm.tasks.master_mode.request_master')
    def test_request_master(self, m_mode_mock):
        cp = test_util.setup_mock(self, ['sys', 'request-master'],
                                  self.adpt, [self.dwrap])
        cp.obj_type.execute()
        m_mode_mock.assert_called_once_with(self.dwrap)

    @mock.patch('pypowervm.tasks.controller_mode.request_controller')
    def test_request_controller(self, m_mode_mock):
        cp = test_util.setup_mock(self, ['sys', 'request-controller'],
                                  self.adpt, [self.dwrap])
        cp.obj_type.execute()
        m_mode_mock.assert_called_once_with(self.dwrap)

    @mock.patch('pypowervm.tasks.management_console.get_public_key',
                return_value="pub_key_str")
    def test_list_public_key(self, m_con_mock):
        cp = test_util.setup_mock(self, ['sys', 'list-public-key'],
                                  self.adpt, [self.dwrap])
        cp.obj_type.execute()
        m_con_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter)
        self.assertEqual("pub_key_str\n", cp.obj_type.cli_out.getvalue())

    @mock.patch('pypowervm.tasks.management_console.add_authorized_key')
    def test_add_auth_key(self, m_con_mock):
        cp = test_util.setup_mock(self, ['sys', 'add-auth-key',
                                         '--key', 'abcd'],
                                  self.adpt, [self.dwrap])
        cp.obj_type.execute()
        m_con_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter, 'abcd')
        m_con_mock.reset_mock()

        cp = test_util.setup_mock(self, ['sys', 'add-auth-key',
                                         '--key', ''],
                                  self.adpt, [self.dwrap])
        self.assertRaises(exc.EmptyKeyError, cp.obj_type.execute)
        m_con_mock.assert_not_called()

    @mock.patch('pypowervm.tasks.management_console.get_authorized_keys')
    def test_list_auth_keys(self, m_con_mock):
        cp = test_util.setup_mock(self, ['sys', 'list-auth-keys'],
                                  self.adpt, [self.dwrap])
        cp.obj_type.execute()
        m_con_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter)

        m_con_mock.reset_mock()
        m_con_mock.return_value = None
        cp = test_util.setup_mock(self, ['sys', 'list-auth-keys'],
                                  self.adpt, [self.dwrap])
        nk_err = self.assertRaises(exc.NoKeysFoundError, cp.obj_type.execute)
        self.assertEqual(nk_err.ret_code, 0)

    def test_rest_log_levels(self):
        cmd = ['sys', 'rest-log-levels']
        some_return = 'foo\n' + ('=' * 57) + '\nbar\n'
        cp = test_util.setup_mock(self, cmd, self.adpt, [self.dwrap])
        m_read = cp.obj_type.ec.sm.adapter.read_by_path
        m_read.return_value.body = some_return

        cp.obj_type.execute()
        m_read.assert_called_with('/rest/api/uom/Debug/SetLoggingLevel')

        cmd.append('--set')
        self.assertRaises(SystemExit, test_util.setup_mock, self, cmd,
                          self.adpt, [self.dwrap])

        cmd.append('no_equals')
        cp = test_util.setup_mock(self, cmd, self.adpt, [self.dwrap])
        self.assertRaises(exc.InvalidKeyValue, cp.obj_type.execute)

        cmd[3] = 'invalid=OFF'
        cp = test_util.setup_mock(self, cmd, self.adpt, [self.dwrap])
        self.assertRaises(exc.InvalidRESTLogType, cp.obj_type.execute)

        cmd[3] = 'ffdc=invalid'
        cp = test_util.setup_mock(self, cmd, self.adpt, [self.dwrap])
        self.assertRaises(exc.InvalidRESTLogLevel, cp.obj_type.execute)

        cmd[3] = 'ffdc=OFF'
        cp = test_util.setup_mock(self, cmd, self.adpt, [self.dwrap])
        m_read = cp.obj_type.ec.sm.adapter.read_by_path
        m_read.return_value.body = some_return
        cp.obj_type.execute()
        m_read.assert_called_with(
            '/rest/api/uom/Debug/SetLoggingLevel?ffdc=OFF')

        cmd.append('audit=FATAL')
        cp = test_util.setup_mock(self, cmd, self.adpt, [self.dwrap])
        m_read = cp.obj_type.ec.sm.adapter.read_by_path
        m_read.return_value.body = some_return
        cp.obj_type.execute()
        m_read.assert_called_with(
            '/rest/api/uom/Debug/SetLoggingLevel?ffdc=OFF&audit=FATAL')
