# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import imp
import logging
import mock
import os
import testtools

import pvmcli.util.config as cfg

bin_dir = os.path.abspath(os.path.join(os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                 os.pardir)), os.pardir)), 'bin'))
pvmctl = imp.load_source('pvmctl',
                         os.path.abspath(os.path.join(bin_dir, 'pvmctl')))


class PVMCTLTest(testtools.TestCase):
    @mock.patch('sys.argv')
    @mock.patch('pvmcli.cli_parser.CLIParser.process', return_value=0)
    @mock.patch('pvmcli.cli_parser.CLIParser.__init__', return_value=None)
    @mock.patch('pwd.getpwnam.pw_uid')
    @mock.patch('pwd.getpwnam')
    @mock.patch('getpass.getuser')
    def test_pvmctl(self, user_mock, pwnam_mock, pw_uid_mock, cli_parser_mock,
                    cli_p_process_mock, argv_mock):
        pvmctl.curr_u_grps = ['notrightgroup']
        self.assertRaises(SystemExit, pvmctl.do_main)
        cli_parser_mock.assert_not_called()
        cli_p_process_mock.assert_not_called()

        pvmctl.curr_u_grps = ['root']
        self.assertRaises(SystemExit, pvmctl.do_main)
        cli_parser_mock.assert_called_once_with(argv_mock[1:])
        cli_parser_mock.reset_mock()

        pvmctl.curr_u_grps = ['pvm_admin']
        self.assertRaises(SystemExit, pvmctl.do_main)
        cli_parser_mock.assert_called_once_with(argv_mock[1:])
        cli_parser_mock.reset_mock()

        logging.basicConfig = mock.Mock(side_effect=Exception())
        pvmctl.GroupWriteRotatingFileHandler = mock.Mock()
        self.assertRaises(SystemExit, pvmctl.do_main)
        cli_parser_mock.assert_called_once_with(argv_mock[1:])
        pvmctl.GroupWriteRotatingFileHandler.assert_not_called()
        logging.basicConfig.assert_called_once_with(
            level=cfg.value('log', 'loglevel'), filename=cfg.value('log',
                                                                   'logfile'),
            format=pvmctl.FORMAT)
