# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock
import testtools

from pypowervm.wrappers.logical_partition import LPAR

import pvmcli.util.session_manager as sess_man
import pvmcli.util.exceptions as exc


class SessionManagerTest(testtools.TestCase):
    @mock.patch('pypowervm.wrappers.logical_partition.LPAR.search')
    def test_split_arg_search(self, lpar_search_mock):
        sm = sess_man.SessionManager()
        sm._session_adapter = mock.Mock()

        sm.split_arg_search('name=foo', LPAR)
        lpar_search_mock.assert_called_with(
            sm.adapter, name='foo', parent_type=None, parent_uuid=None)
        lpar_search_mock.reset_mock()

        self.assertRaises(exc.InvalidSplitArgError, sm.split_arg_search,
                          'name!foo', LPAR)
        lpar_search_mock.reset_mock()

        sm.split_arg_search(
            'name=foo', LPAR, parent_type='bar',
            parent_uuid='7C5B5358-EC25-47BC-8000-D377BB0ED310')
        lpar_search_mock.assert_called_with(
            sm.adapter, name='foo', parent_type='bar',
            parent_uuid='7C5B5358-EC25-47BC-8000-D377BB0ED310')
