# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import pvmcli.command.shared_storage_pool as ssp_c
import pvmcli.help.help_shared_storage_pool as help_ssp
import pvmcli.tests.util.test_util as test_util

import pypowervm.tests.test_utils.test_wrapper_abc as twa
import pypowervm.wrappers.storage as stor


LIST_USAGE_HELP = (
    ["usage: pvmctl [help | --help] SharedStoragePool list {--object-id | "
     "-i} <SharedStoragePool field>=value {args}"])

DISPLAY_FIELD_HELP = (
    ["--display-fields options:\n"
     "    LogicalUnit.capacity\n"
     "    LogicalUnit.cloned_from_udid\n"
     "    LogicalUnit.emulate_model\n"
     "    LogicalUnit.in_use\n"
     "    LogicalUnit.is_thin\n"
     "    LogicalUnit.lu_type\n"
     "    LogicalUnit.name\n"
     "    LogicalUnit.tag\n"
     "    LogicalUnit.udid\n"
     "    PhysicalVolume.avail_for_use\n"
     "    PhysicalVolume.capacity\n"
     "    PhysicalVolume.description\n"
     "    PhysicalVolume.emulate_model\n"
     "    PhysicalVolume.is_fc_backed\n"
     "    PhysicalVolume.loc_code\n"
     "    PhysicalVolume.name\n"
     "    PhysicalVolume.read_iops_limit\n"
     "    PhysicalVolume.state\n"
     "    PhysicalVolume.tag\n"
     "    PhysicalVolume.udid\n"
     "    PhysicalVolume.write_iops_limit\n"
     "    SharedStoragePool.capacity\n"
     "    SharedStoragePool.free_space\n"
     "    SharedStoragePool.logical_units\n"
     "    SharedStoragePool.name\n"
     "    SharedStoragePool.over_commit_space\n"
     "    SharedStoragePool.physical_volumes\n"
     "    SharedStoragePool.total_lu_size\n"
     "    SharedStoragePool.udid\n"
     "    SharedStoragePool.uuid\n"
     ""])


class SharedStoragePoolTest(twa.TestWrapper):
    ''' Test framework for SharedStoragePool Object Type. '''
    @property
    def file(self):
        return 'ssp.txt'

    @property
    def wrapper_class_to_test(self):
        return stor.SSP

    def test_shared_storage_pool(self):
        ssp = ssp_c.SharedStoragePool()

        test_util.test_obj_type(self, ssp, 'SharedStoragePool', stor.SSP,
                                field_models=[stor.SSP, stor.LU, stor.PV],
                                name_to_ops={'list'},
                                exp_help=help_ssp.help_text)

        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={ssp.name_to_ops['list']: None},
            child_model_ops={ssp.name_to_ops['list']: None},
            elem_wrp_func_ops={ssp.name_to_ops['list']: None})

    def test_list(self):
        test_util.test_list(
            self, 'SharedStoragePool',
            LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
            ["Shared Storage Pools\n"
             "+---------+----------+------------+----------+\n"
             "|   Name  | Cap (GB) | Free Space | OC Space |\n"
             "+---------+----------+------------+----------+\n"
             "| neossp1 |  49.88   |   48.98    | 1.234567 |\n"
             "+---------+----------+------------+----------+\n"],
            ["---------------------\n"
             "| SharedStoragePool |\n"
             "---------------------\n"
             "  capacity=49.88\n"
             "  free_space=48.98\n"
             "  name=neossp1\n"
             "  over_commit_space=1.234567\n"
             "  total_lu_size=1.0\n"
             "  udid=24cfc907d2abf511e4b2d540f2e95daf30000000000972FB37000"
             "0000054D14EB8\n"
             "  uuid=e357a79a-7a3d-35b6-8405-55ab6a2d0de7\n"
             "  ---------------\n"
             "  | LogicalUnit |\n"
             "  ---------------\n"
             "    capacity=1.0\n"
             "    emulate_model=True\n"
             "    in_use=True\n"
             "    is_thin=True\n"
             "    lu_type=VirtualIO_Disk\n"
             "    name=neolu1\n"
             "    udid=27cfc907d2abf511e4b2d540f2e95daf301a02b0904778d755d"
             "f5a46fe25e500d8\n"
             "  ------------------\n"
             "  | PhysicalVolume |\n"
             "  ------------------\n"
             "    avail_for_use=False\n"
             "    capacity=51200\n"
             "    description=MPIO IBM 2076 FC Disk\n"
             "    emulate_model=True\n"
             "    is_fc_backed=True\n"
             "    name=hdisk3\n"
             "    state=active\n"
             "    udid=01M0lCTTIxNDUxMjQ2MDA1MDc2ODAyODI4NjFEODgwMDAwMDAwMD"
             "AwMDAwMw==\n"],
            ['<SharedStoragePool:SharedStoragePool xmlns:SharedStoragePool="'],
            ['SharedStoragePool.name', 'LogicalUnit.capacity',
             'PhysicalVolume.name'],
            ["name=neossp1,capacity=1.0,name=hdisk3\n"],
            ["neossp1,1.0,hdisk3\n"],
            ["name___neossp1,capacity___1.0,name___hdisk3\n"],
            ["name=neossp1___capacity=1.0___name=hdisk3\n"],
            ["name=neossp1,capacity=1.0,name=hdisk3\n"],
            ["name___neossp1___capacity___1.0___name___hdisk3\n"],
            'SharedStoragePool')
