# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2016
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock

import pvmcli.command.virtual_media_repository as vmr_c
import pvmcli.help.help_virtual_media_repository as help_vmr
import pvmcli.tests.util.test_util as test_util

import pypowervm.tests.test_utils.test_wrapper_abc as twa
import pypowervm.wrappers.storage as stor
from pypowervm.wrappers import virtual_io_server as v


LIST_USAGE_HELP = (
    ["usage: pvmctl [help | --help] VirtualMediaRepository list {--parent-id "
     "| -p} <VirtualIOServer field>=value {--child-id | -c} <VolumeGroup "
     "field>=value {--object-id | -i} <VirtualMediaRepository field>="
     "value {args}"])

DISPLAY_FIELD_HELP = (
    ["--display-fields options:\n"
     "    VirtualMediaRepository.name\n"
     "    VirtualMediaRepository.optical_media\n"
     "    VirtualMediaRepository.size\n"
     "    VirtualOpticalMedia.media_name\n"
     "    VirtualOpticalMedia.mount_type\n"
     "    VirtualOpticalMedia.name\n"
     "    VirtualOpticalMedia.size\n"
     "    VirtualOpticalMedia.udid\n"])


class VirtualMediaRepositoryTest(twa.TestWrapper):
    ''' Test framework for VirtualMediaRepository Object Type. '''
    @property
    def file(self):
        return 'fake_volume_group.txt'

    @property
    def wrapper_class_to_test(self):
        return stor.VMediaRepos

    def test_virtual_media_repository(self):
        vmr = vmr_c.VirtualMediaRepository()

        test_util.test_obj_type(self, vmr, 'VirtualMediaRepository',
                                stor.VMediaRepos,
                                field_models=[stor.VMediaRepos,
                                              stor.VOptMedia],
                                name_to_ops={'list'},
                                exp_help=help_vmr.help_text)

        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={vmr.name_to_ops['list']: [v.VIOS]},
            child_model_ops={vmr.name_to_ops['list']: [stor.VG]},
            elem_wrp_func_ops={vmr.name_to_ops['list']:
                               [stor.VG.vmedia_repos]})

    def test_help(self):
        ''' Test help arguments for an object type. '''
        test_util.test_help(self, 'VirtualMediaRepository')

    @mock.patch("pypowervm.wrappers.virtual_io_server.VIOS.search")
    def test_list(self, vios_search_mock):
        sample_vios_w = test_util.mock_vios(self.adpt)
        vios_search_mock.return_value = [sample_vios_w]
        vg_wrap = stor.VG.wrap(self.resp)
        test_util.test_list(
            self, 'VirtualMediaRepository',
            LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
            ["Virtual Media Repositories\n"
             "+-------------+------------+---------------+-----------+"
             "---------------+--------------+----+\n"
             "|     VIOS    |     VG     | VMR Size (GB) | Free (GB) |"
             "    OM Name    | OM Size (GB) | rw |\n"
             "+-------------+------------+---------------+-----------+"
             "---------------+--------------+----+\n"
             "| vios_1_Drew | image_pool |      11.0     |  10.8535  |"
             "  blank_media1 |    0.0977    | rw |\n"
             "|             |            |               |           |"
             " blank_media_2 |    0.0488    | rw |\n"
             "+-------------+------------+---------------+-----------+"
             "---------------+--------------+----+\n"],
            ["--------------------------\n"
             "| VirtualMediaRepository |\n"
             "--------------------------\n"
             "  name=VMLibrary\n"
             "  size=11.0\n"
             "  -----------------------\n"
             "  | VirtualOpticalMedia |\n"
             "  -----------------------\n"
             "    media_name=blank_media1\n"
             "    mount_type=rw\n"
             "    name=blank_media1\n"
             "    size=0.0977\n"
             "    udid=0eblank_media1\n"
             "  -----------------------\n"
             "  | VirtualOpticalMedia |\n"
             "  -----------------------\n"
             "    media_name=blank_media_2\n"
             "    mount_type=rw\n"
             "    name=blank_media_2\n"
             "    size=0.0488\n"
             "    udid=0eblank_media_2\n"],
            ['VirtualMediaRepository xmlns="http://www.ibm.com/xmlns/systems'],
            ['VirtualMediaRepository.size', 'VirtualOpticalMedia.media_name'],
            ["size=11.0,media_name=blank_media1\n"
             "size=11.0,media_name=blank_media_2\n"],
            ["11.0,blank_media1\n11.0,blank_media_2\n"],
            ["size___11.0,media_name___blank_media1\n"
             "size___11.0,media_name___blank_media_2\n"],
            ["size=11.0___media_name=blank_media1\n"
             "size=11.0___media_name=blank_media_2\n"],
            ["size=11.0,media_name=blank_media1___"
             "size=11.0,media_name=blank_media_2"],
            ["size___11.0___media_name___blank_media1___"
             "size___11.0___media_name___blank_media_2\n"],
            {'root_type': 'VirtualIOServer', 'child_type': 'VolumeGroup',
             'root_id': '014C1F78-B210-40B2-A12E-C82316B878CA'},
            c_wrps=[vg_wrap], wrps=[vg_wrap.vmedia_repos[0]],
            p_wrps=[sample_vios_w])
