# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

import textwrap

from pvmcli.util.i18n import _


HELP_WRAP_WIDTH = 56

# Help
HELP_DESC = _('show this help message and exit')


# Authentication
USER = _("User for remote execution authentication. "
         "Overrides PVM_USER environment variable.")
PASSWORD = _("Password for remote execution authentication. "
             "Overrides PVM_PASS environment variable.")
HOST = _("Host IP for remote execution authentication. "
         "Overrides PVM_HOST environment variable.")
PORT = _("Port for remote execution authentication. "
         "Overrides PVM_PORT environment variable.")

# UUIDs
UUID = _('Unique id of specified object type')
ROOT_UUID = _('Unique id of object type\'s parent root type')
CHILD_UUID = _('Unique id of object type\'s parent child type')

# Fields
DISPLAY_FIELDS = _('Fields to output after a transaction is complete.')
SET_FIELDS = _('Fields to set for a transaction.')
HELP_ID_ARG = _("Identifier used when querying for an object type. Specified "
                "as a name=value pair. Example: --object-id name=value")
HELP_PARENT_ID_ARG = _("Identifier of an object type's root level parent "
                       "object used when querying for an object type. "
                       "Specified as a name=value pair. See usage for valid "
                       "types. Example: --parent-id name=value")
HELP_CHILD_ID_ARG = _("Identifier of an object type's child level parent "
                      "object used when querying for an object type. "
                      "Specified as a name=value pair. See usage for valid "
                      "types. Example: --child-id name=value")
XML = _('XML file to pass for processing.')

# Output
FIELD_DELIM = _(r"String to separate output of display fields within a "
                r"wrapper response. To specify a newline as the field "
                r"delimiter, pass '\n' as the argument. Default: ,")
RECORD_DELIM = _(r"String to separate output of different wrapper responses. "
                 r"To specify a newline as the record delimiter, pass '\n' "
                 r"as the argument. Default: New line")
LABEL_CHAR = _(r"String to separate label from value in output. "
               r"To specify a newline as the label character, pass '\n' "
               r"as the argument. Default: =")
MAX_WIDTH = _(r"Integer value representing the maximum width of output. "
              r"Default: 80")
HIDE_LABEL = _("On non-xml output, hide labels for each value. "
               "Default: False.")
XML_DISP = _('Display raw response XML for all query responses.')
ALL_DISP = _("Display raw output for all wrappers, child elements and atomic "
             "values for all query responses. This will likely be a very "
             "large amount of output.")
WHERE_DISP = _('Filters output based on provided equality strings. Each '
               'argument takes the following form: '
               '"<display_field>=/!=<value>". Note that each value must '
               'be surrounded in ". To only output wrappers that contain a '
               'display field with a specified value, use =. Conversely, to '
               'output all wrappers that do not contain a value, use !=.\n'
               'Example: pvmctl ManagedSystem list -d IOSlot.description '
               ' --where "IOSlot.pci_class=4" "IOSlot.description!=Empty '
               'slot" will output all ManagedSystem IOSlot descriptions '
               'such that the IOSlot pci_class equals 4 AND the IOSlot '
               'description is not "Empty slot".')
NOPROMPT_HELP = _("Assume a positive response to all prompted input.")

# Upload
UPLOAD_VIOS_HELP = _("Name of the Virtual I/O Server through which to "
                     "perform the upload. If not specified, default to "
                     "the first VIOS.")
UPLOAD_DATA_HELP = _("(OPTIONAL) The path to the file to upload. If not "
                     "specified, read from stdin.")
UPLOAD_DATA_SIZE_HELP = _("(OPTIONAL) The size of the file (in bytes). "
                          "Required if data is passed from stdin, optional "
                          "if data argument is a file.")
UPLOAD_NAME_HELP = _("The name that should be given to the new storage "
                     "element.")
UPLOAD_SIZE_HELP = _("(OPTIONAL) The size of the storage element (in bytes). "
                     "Not required if it should match the file.  Must be at "
                     "least as large as the file.")
UPLOAD_CHKSUM_HELP = _("(OPTIONAL) The SHA256 checksum for the file.  Useful "
                       "for integrity checks.")
UPLOAD_SSP_HELP = _("Name of the Shared Storage Pool on which to create the "
                    "new storage element.")
UPLOAD_VG_HELP = _("Name of the Volume Group on which to create the new "
                   "storage element.")
PROG_NAME = _('pvmctl')
PROG_DESC = _('PowerVM CLI')
HELP_ARGS = _('[help | --help]')
VERSION_ARGS = _('[--version]')
OBJ_TYPE_USG = _('{ObjectType {operation [args]}}')
OP_USG = _('{operation [args]}')


def wrap_help(txt):
    ''' argparse indent before start of argument description is 24 characters.
        We are limiting to 80 characters per line, so we need to wrap 56
        characters for each help description. To allow for new lines in the
        help description, turn off dropping of whitespace.
    '''
    return textwrap.fill(txt, width=HELP_WRAP_WIDTH, replace_whitespace=False,
                         drop_whitespace=False)
